@@ -27,13 +27,23 @@
 #include <stdarg.h>
 #include "RawListener.h"
 
+#ifdef _U
+#undef _U
+#endif
+
 #define _U(M, L) \
+	m_atLeastOneCallback = true; \
 	if (!m_printCallgraphScore) \
 			__iuprintf M; \
 	else \
 		m_callStack.push(L);
 
+#ifdef _D
+#undef _D
+#endif
+
 #define _D(M, L) \
+	m_atLeastOneCallback = true; \
 	if (!m_printCallgraphScore) \
 			__idprintf M; \
 	else \
@@ -47,18 +57,21 @@
 RawListenerImpl::RawListenerImpl(bool printCallgraphScore) :
 	m_indent(0),
 	m_callbackMisses(0),
-	m_printCallgraphScore(printCallgraphScore)
+	m_atLeastOneCallback(false),
+	m_printCallgraphScore(printCallgraphScore),
+	m_callStack()
 {
 }
 
 RawListenerImpl::~RawListenerImpl()
 {
 	if (m_printCallgraphScore)
-		printf("%d\n", (int)(m_callStack.size() + m_callbackMisses));
+		printf("%d\n", m_atLeastOneCallback ? (int)(m_callStack.size() + m_callbackMisses) : -1);
 }
 
 void RawListenerImpl::__iprintf(const char *format, ...)
 {
+	m_atLeastOneCallback = true;
 	if (m_printCallgraphScore) return;
 	
 	va_list args;
@@ -71,6 +84,7 @@ void RawListenerImpl::__iprintf(const char *format, ...)
 
 void RawListenerImpl::__iuprintf(const char *format, ...)
 {
+	m_atLeastOneCallback = true;
 	va_list args;
 	va_start(args, format);
 	for (int i=0; i<m_indent; i++)
@@ -82,6 +96,7 @@ void RawListenerImpl::__iuprintf(const char *format, ...)
 
 void RawListenerImpl::__idprintf(const char *format, ...)
 {
+	m_atLeastOneCallback = true;
 	va_list args;
 	va_start(args, format);
 	__indentDown();
