@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2004, 2005 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2004, 2005 William Lachance (wrlach@gmail.com)
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -28,10 +28,16 @@
 #include <gsf/gsf-infile-msole.h>
 #include <gsf/gsf-input.h>
 
+/* THIS CLASS IS DEPRECATED
+   Use the WPXFileStream or ()Stream classes
+   from WPXStreamImplementation.cpp instead
+   THIS CLASS WILL BE REMOVED IN NEXT RELEASE CYCLE
+ */
+
 GSFInputStream::GSFInputStream(GsfInput *input) :
 	WPXInputStream(true),
 	m_input(input),
-	m_ole(NULL)
+	m_ole(0)
 {
 	g_object_ref(G_OBJECT(input));
 }
@@ -46,11 +52,14 @@ GSFInputStream::~GSFInputStream()
 
 const uint8_t * GSFInputStream::read(size_t numBytes, size_t &numBytesRead)
 {
-	const uint8_t *buf = gsf_input_read(m_input, numBytes, NULL);
+	numBytesRead = 0;
+	
+	if (0 == numBytes)
+		return 0;
 
-	if (buf == NULL)
-		numBytesRead = 0;
-	else
+	const uint8_t *buf = gsf_input_read(m_input, numBytes, 0);
+
+	if (buf)
 		numBytesRead = numBytes;
 
 	return buf;
@@ -65,6 +74,7 @@ int GSFInputStream::seek(long offset, WPX_SEEK_TYPE seekType)
 		gsfSeekType = G_SEEK_CUR;
 		break;
 	case WPX_SEEK_SET:
+	default:
 		gsfSeekType = G_SEEK_SET;
 		break;
 	}
@@ -75,9 +85,9 @@ int GSFInputStream::seek(long offset, WPX_SEEK_TYPE seekType)
 bool GSFInputStream::isOLEStream()
 {
 	if (!m_ole)
-		m_ole = GSF_INFILE(gsf_infile_msole_new (m_input, NULL)); 
+		m_ole = GSF_INFILE(gsf_infile_msole_new (m_input, 0)); 
 
-	if (m_ole != NULL)
+	if (m_ole)
 		return true;
 
 	return false;
@@ -85,10 +95,10 @@ bool GSFInputStream::isOLEStream()
 
 WPXInputStream * GSFInputStream::getDocumentOLEStream()
 {
-	WPXInputStream *documentStream = NULL;
+	WPXInputStream *documentStream = 0;
 
 	if (!m_ole)
-		m_ole = GSF_INFILE(gsf_infile_msole_new (m_input, NULL)); 
+		m_ole = GSF_INFILE(gsf_infile_msole_new (m_input, 0)); 
 
 	if (m_ole)
 	{
