@@ -29,20 +29,20 @@
 #include "libwpd_internal.h"
 #include "WP3SubDocument.h"
 #include <algorithm>
+#include <limits>
 
 _WP3ContentParsingState::_WP3ContentParsingState():
 	m_colSpan(1),
 	m_rowSpan(1),
-	m_cellFillColor(0)
+	m_textBuffer(),
+	m_cellFillColor(0),
+	m_noteReference(),
+	m_tableList()
 {
-	m_textBuffer.clear();
-	m_noteReference.clear();
 }
 
 _WP3ContentParsingState::~_WP3ContentParsingState()
 {
-	m_textBuffer.clear();
-	m_noteReference.clear();
 	DELETEP(m_cellFillColor);
 }
 
@@ -74,15 +74,57 @@ void WP3ContentListener::insertCharacter(const uint16_t character)
 	}
 }
 
-void WP3ContentListener::insertTab(const uint8_t /* tabType */, float /* tabPosition */)
+void WP3ContentListener::insertTab()
 {
         if (!isUndoOn())
 	{
-		if (!m_ps->m_isSpanOpened)
-			_openSpan();
+		if (!m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
+		{
+			if (m_ps->m_tabStops.empty() || (_getNextTabStop() == (std::numeric_limits<float>::max)()))
+				m_ps->m_textIndentByTabs += 0.5f;
+			else
+				m_ps->m_textIndentByTabs = _getNextTabStop() - (m_ps->m_leftMarginByTabs  + m_ps->m_textIndentByParagraphIndentChange);
+			m_ps->m_paragraphTextIndent = m_ps->m_textIndentByParagraphIndentChange
+				+ m_ps->m_textIndentByTabs;
+			m_ps->m_paragraphMarginLeft = m_ps->m_leftMarginByPageMarginChange
+				+ m_ps->m_leftMarginByParagraphMarginChange + m_ps->m_leftMarginByTabs;
+			m_ps->m_paragraphMarginRight = m_ps->m_rightMarginByPageMarginChange
+				+ m_ps->m_rightMarginByParagraphMarginChange + m_ps->m_rightMarginByTabs;
+			
+			m_ps->m_listReferencePosition = m_ps->m_paragraphMarginLeft + m_ps->m_paragraphTextIndent;
+		}
 		else
-			_flushText();
-		m_listenerImpl->insertTab();
+		{
+			if (!m_ps->m_isSpanOpened)
+				_openSpan();
+			else
+				_flushText();
+			m_listenerImpl->insertTab();
+		}
+	}
+}
+
+void WP3ContentListener::insertTab(const uint8_t tabType, const float /* tabPosition */)
+{
+	if (!isUndoOn())
+	{
+		if (!m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
+		{
+			switch (tabType)
+			{
+			case WP3_TAB_GROUP_CENTER:
+				m_ps->m_tempParagraphJustification = WPX_PARAGRAPH_JUSTIFICATION_CENTER;
+				return;
+			case WP3_TAB_GROUP_FLUSH_RIGHT:
+				m_ps->m_tempParagraphJustification = WPX_PARAGRAPH_JUSTIFICATION_RIGHT;
+				return;
+			default:
+				insertTab();
+				return;
+			}
+		}
+		else
+			insertTab();
 	}
 }
 
@@ -140,8 +182,8 @@ void WP3ContentListener::defineTable(const uint8_t position, const uint16_t left
 		m_ps->m_tableDefinition.m_leftOffset = _movePositionToFirstColumn( (float)((double)leftOffset / (double)WPX_NUM_WPUS_PER_INCH) ) - m_ps->m_paragraphMarginLeft;
 
 		// remove all the old column information
-		m_ps->m_tableDefinition.columns.clear();
-		m_ps->m_tableDefinition.columnsProperties.clear();
+		m_ps->m_tableDefinition.m_columns.clear();
+		m_ps->m_tableDefinition.m_columnsProperties.clear();
 		m_ps->m_numRowsToSkip.clear();
 	}
 }
@@ -158,13 +200,13 @@ void WP3ContentListener::addTableColumnDefinition(const uint32_t width, const ui
 		colDef.m_rightGutter = (float)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
 
 		// add the new column definition to our table definition
-		m_ps->m_tableDefinition.columns.push_back(colDef);
+		m_ps->m_tableDefinition.m_columns.push_back(colDef);
 		
 		WPXColumnProperties colProp;
 		colProp.m_attributes = attributes;
 		colProp.m_alignment = alignment;
 		
-		m_ps->m_tableDefinition.columnsProperties.push_back(colProp);
+		m_ps->m_tableDefinition.m_columnsProperties.push_back(colProp);
 		
 		// initialize the variable that tells us how many columns to skip
 		m_ps->m_numRowsToSkip.push_back(0);
@@ -449,6 +491,8 @@ void WP3ContentListener::columnChange(const WPXTextColumnType /* columnType */,
 {
 	if (!isUndoOn())
 	{
+		_openPageSpan();
+
 		int oldColumnNum = m_ps->m_numColumns;
 
 		// In WP, the last column ends with a hard column break code.
@@ -508,10 +552,12 @@ void WP3ContentListener::columnChange(const WPXTextColumnType /* columnType */,
 		if ((m_ps->m_numColumns > 1 && oldColumnNum <= 1) || (m_ps->m_numColumns <= 1 && oldColumnNum > 1))
 		{
 			m_ps->m_paragraphMarginLeft -= m_ps->m_leftMarginByPageMarginChange;
+			m_ps->m_listReferencePosition -= m_ps->m_leftMarginByPageMarginChange;
 			m_ps->m_paragraphMarginRight -= m_ps->m_rightMarginByPageMarginChange;
 			std::swap(m_ps->m_leftMarginByPageMarginChange, m_ps->m_sectionMarginLeft);
 			std::swap(m_ps->m_rightMarginByPageMarginChange, m_ps->m_sectionMarginRight);
 			m_ps->m_paragraphMarginLeft += m_ps->m_leftMarginByPageMarginChange;
+			m_ps->m_listReferencePosition += m_ps->m_leftMarginByPageMarginChange;
 			m_ps->m_paragraphMarginRight += m_ps->m_rightMarginByPageMarginChange;
 		}
 	}
@@ -570,7 +616,13 @@ void WP3ContentListener::insertNote(const WPXNoteType noteType, WP3SubDocument *
 {
 	if (!isUndoOn() && !m_ps->m_isNote)
 	{
-		_closeSpan();
+		if (!m_ps->m_isParagraphOpened)
+			_openParagraph();
+		else
+		{
+			_flushText();
+			_closeSpan();
+		}
 		m_ps->m_isNote = true;
 		WPXNumberingType numberingType = _extractWPXNumberingTypeFromBuf(m_parseState->m_noteReference, ARABIC);
 		int number = _extractDisplayReferenceNumberFromBuf(m_parseState->m_noteReference, numberingType);
@@ -594,6 +646,142 @@ void WP3ContentListener::insertNote(const WPXNoteType noteType, WP3SubDocument *
 	}
 }
 
+void WP3ContentListener::backTab()
+{
+	if (!isUndoOn() && !m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
+	{
+		if (m_ps->m_tabStops.empty() || (_getPreviousTabStop() == (std::numeric_limits<float>::max)()))
+			m_ps->m_textIndentByTabs -= 0.5f;
+		else
+			m_ps->m_textIndentByTabs = _getPreviousTabStop() - (m_ps->m_leftMarginByTabs  + m_ps->m_textIndentByParagraphIndentChange);
+
+		m_ps->m_paragraphTextIndent = m_ps->m_textIndentByParagraphIndentChange
+			+ m_ps->m_textIndentByTabs;
+		m_ps->m_paragraphMarginLeft = m_ps->m_leftMarginByPageMarginChange
+			+ m_ps->m_leftMarginByParagraphMarginChange + m_ps->m_leftMarginByTabs;
+		m_ps->m_paragraphMarginRight = m_ps->m_rightMarginByPageMarginChange
+			+ m_ps->m_rightMarginByParagraphMarginChange + m_ps->m_rightMarginByTabs;
+			
+		m_ps->m_listReferencePosition = m_ps->m_paragraphMarginLeft + m_ps->m_paragraphTextIndent;
+	}
+}
+
+void WP3ContentListener::leftIndent()
+{
+	if (!isUndoOn())
+	{
+		if (!m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
+		{
+			if (m_ps->m_tabStops.empty() || (_getNextTabStop() == (std::numeric_limits<float>::min)()))
+				m_ps->m_leftMarginByTabs += 0.5f;
+			else
+				m_ps->m_leftMarginByTabs = _getNextTabStop() - (m_ps->m_textIndentByTabs + m_ps->m_textIndentByParagraphIndentChange);
+
+			if (m_ps->m_paragraphTextIndent != 0.0f)
+				m_ps->m_textIndentByTabs -= m_ps->m_paragraphTextIndent;
+
+			m_ps->m_paragraphTextIndent = m_ps->m_textIndentByParagraphIndentChange
+				+ m_ps->m_textIndentByTabs;
+			m_ps->m_paragraphMarginLeft = m_ps->m_leftMarginByPageMarginChange
+				+ m_ps->m_leftMarginByParagraphMarginChange + m_ps->m_leftMarginByTabs;
+			m_ps->m_paragraphMarginRight = m_ps->m_rightMarginByPageMarginChange
+				+ m_ps->m_rightMarginByParagraphMarginChange + m_ps->m_rightMarginByTabs;
+			
+			m_ps->m_listReferencePosition = m_ps->m_paragraphMarginLeft + m_ps->m_paragraphTextIndent;
+		}
+		else
+			insertTab();
+	}
+}
+
+void WP3ContentListener::leftIndent(const float offset)
+{
+	if (!isUndoOn())
+	{
+		if (!m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
+		{
+			if (offset == 0.0f)
+				m_ps->m_leftMarginByTabs += 0.5f;
+			else
+				m_ps->m_leftMarginByTabs += (offset / 72.0f);
+
+			if (m_ps->m_paragraphTextIndent != 0.0f)
+				m_ps->m_textIndentByTabs -= m_ps->m_paragraphTextIndent;
+
+			m_ps->m_paragraphTextIndent = m_ps->m_textIndentByParagraphIndentChange
+				+ m_ps->m_textIndentByTabs;
+			m_ps->m_paragraphMarginLeft = m_ps->m_leftMarginByPageMarginChange
+				+ m_ps->m_leftMarginByParagraphMarginChange + m_ps->m_leftMarginByTabs;
+			m_ps->m_paragraphMarginRight = m_ps->m_rightMarginByPageMarginChange
+				+ m_ps->m_rightMarginByParagraphMarginChange + m_ps->m_rightMarginByTabs;
+			
+			m_ps->m_listReferencePosition = m_ps->m_paragraphMarginLeft + m_ps->m_paragraphTextIndent;
+		}
+		else
+			insertTab();
+	}
+}
+
+void WP3ContentListener::leftRightIndent()
+{
+	if (!isUndoOn())
+	{
+		if (!m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
+		{
+			if (m_ps->m_tabStops.empty() || (_getNextTabStop() == (std::numeric_limits<float>::min)()))
+				m_ps->m_leftMarginByTabs += 0.5f;
+			else
+				m_ps->m_leftMarginByTabs = _getNextTabStop() - (m_ps->m_textIndentByTabs + m_ps->m_textIndentByParagraphIndentChange);
+			
+			if (m_ps->m_paragraphTextIndent != 0.0f)
+				m_ps->m_textIndentByTabs -= m_ps->m_paragraphTextIndent;
+
+			m_ps->m_rightMarginByTabs = m_ps->m_leftMarginByTabs;
+
+			m_ps->m_paragraphTextIndent = m_ps->m_textIndentByParagraphIndentChange
+				+ m_ps->m_textIndentByTabs;
+			m_ps->m_paragraphMarginLeft = m_ps->m_leftMarginByPageMarginChange
+				+ m_ps->m_leftMarginByParagraphMarginChange + m_ps->m_leftMarginByTabs;
+			m_ps->m_paragraphMarginRight = m_ps->m_rightMarginByPageMarginChange
+				+ m_ps->m_rightMarginByParagraphMarginChange + m_ps->m_rightMarginByTabs;
+			
+			m_ps->m_listReferencePosition = m_ps->m_paragraphMarginLeft + m_ps->m_paragraphTextIndent;
+		}
+		else
+			insertTab();
+	}
+}
+
+void WP3ContentListener::leftRightIndent(const float offset)
+{
+	if (!isUndoOn())
+	{
+		if (!m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
+		{
+			if (offset == 0.0f)
+				m_ps->m_leftMarginByTabs += 0.5f;
+			else
+				m_ps->m_leftMarginByTabs += (offset / 72.0f);
+
+			if (m_ps->m_paragraphTextIndent != 0.0f)
+				m_ps->m_textIndentByTabs -= m_ps->m_paragraphTextIndent;
+
+			m_ps->m_rightMarginByTabs = m_ps->m_leftMarginByTabs;
+
+			m_ps->m_paragraphTextIndent = m_ps->m_textIndentByParagraphIndentChange
+				+ m_ps->m_textIndentByTabs;
+			m_ps->m_paragraphMarginLeft = m_ps->m_leftMarginByPageMarginChange
+				+ m_ps->m_leftMarginByParagraphMarginChange + m_ps->m_leftMarginByTabs;
+			m_ps->m_paragraphMarginRight = m_ps->m_rightMarginByPageMarginChange
+				+ m_ps->m_rightMarginByParagraphMarginChange + m_ps->m_rightMarginByTabs;
+			
+			m_ps->m_listReferencePosition = m_ps->m_paragraphMarginLeft + m_ps->m_paragraphTextIndent;
+		}
+		else
+			insertTab();
+	}
+}
+
 void WP3ContentListener::_handleSubDocument(const WPXSubDocument *subDocument, const bool isHeaderFooter,
 						WPXTableList /* tableList */, int /* nextTableIndice */)
 {
