@@ -32,7 +32,7 @@ extern int WP3_FIXED_LENGTH_FUNCTION_GROUP_SIZE[16];
 /* Main function group list  */
 
 #define WP3_EXTENDED_CHARACTER_GROUP 0xC0
-
+#define WP3_TAB_GROUP 0xC1
 #define WP3_INDENT_GROUP 0xC2
 #define WP3_ATTRIBUTE_GROUP 0xC3
 
@@ -53,6 +53,19 @@ extern int WP3_FIXED_LENGTH_FUNCTION_GROUP_SIZE[16];
 
 #define WP3_TABLES_GROUP 0xE2
 
+/* Tab Group */
+#define WP3_TAB_GROUP_TAB 0x00
+#define WP3_TAB_GROUP_CENTER 0x01
+#define WP3_TAB_GROUP_FLUSH_RIGHT 0x02
+#define WP3_TAB_GROUP_BACK_TAB 0x03
+#define WP3_TAB_GROUP_CHARACTER_KERNING 0x04
+#define WP3_TAB_GROUP_BAR_TAB 0x05
+#define WP3_TAB_GROUP_FIXED_TAB 0x06
+
+/* Indent Group */
+#define WP3_INDENT_GROUP_LEFT_INDENT 0x00
+#define WP3_INDENT_GROUP_LEFT_RIGHT_INDENT 0x01
+
 /* Attribute Group */
 #define WP3_ATTRIBUTE_BOLD 0
 #define WP3_ATTRIBUTE_ITALICS 1
