@@ -1,5 +1,6 @@
 /* libwpd
- * Copyright (C) 2004 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2004 Marc Maurer (uwog@uwog.net)
+ * Copyright (C) 2006 Fridrich Strba (fridrich.strba@bluewin.ch)
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -13,7 +14,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  * For further information visit http://libwpd.sourceforge.net
  */
@@ -25,19 +26,18 @@
 #ifndef WP3PART_H
 #define WP3PART_H
 
-#include "WPXPart.h"
-#include "WPXParser.h"
-#include "WP3HLListener.h"
+#include "libwpd_types.h"
 
-class WP3LLListener;
+class WPXInputStream;
+class WP3Listener;
 
 class WP3Part
 {
  public:
 	virtual ~WP3Part() {}
 	
-	static WP3Part * constructPart(WPXInputStream *input, uint8_t readVal);
-	virtual void parse(WP3HLListener *listener) = 0;
+	static WP3Part * constructPart(WPXInputStream *input, const uint8_t readVal);
+	virtual void parse(WP3Listener *listener) = 0;
 };
 
-#endif /* WP5PART_H */
+#endif /* WP3PART_H */
