@@ -40,7 +40,8 @@ public:
 
 	void startDocument() {}
 	void insertCharacter(const uint16_t /* character */) { if (!isUndoOn()) m_currentPageHasContent = true; }
-	void insertTab(const uint8_t /* tabType */, float /* tabPosition */) { if (!isUndoOn()) m_currentPageHasContent = true; }
+	void insertTab() { if (!isUndoOn()) m_currentPageHasContent = true; }
+	void insertTab(const uint8_t /* tabType */, const float /* tabPosition */) { if (!isUndoOn()) m_currentPageHasContent = true; }
 	void insertEOL() { if (!isUndoOn()) m_currentPageHasContent = true; }
  	void insertBreak(const uint8_t breakType);
 	void attributeChange(const bool /* isOn */, const uint8_t /* attribute */) {}
@@ -75,11 +76,18 @@ public:
 	void insertNote(const WPXNoteType /* noteType */, WP3SubDocument * /* subDocument */) { if (!isUndoOn()) m_currentPageHasContent = true; }
 	void headerFooterGroup(const uint8_t headerFooterType, const uint8_t occurenceBits, WP3SubDocument *subDocument);
 	void suppressPage(const uint16_t suppressCode);
+	void backTab() { if (!isUndoOn()) m_currentPageHasContent = true; }
+	void leftIndent() {}
+	void leftIndent(const float /* offset */) {}
+	void leftRightIndent() {}
+	void leftRightIndent(const float /* offset */) {}
 
 protected:
 	void _handleSubDocument(const WPXSubDocument *subDocument, const bool isHeaderFooter, WPXTableList tableList, int nextTableIndice = 0);
 
 private:
+	WP3StylesListener(const WP3StylesListener&);
+	WP3StylesListener& operator=(const WP3StylesListener&);
 	WPXPageSpan m_currentPage;
 
 	WPXTableList m_tableList;
