@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2003 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2003 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2003 Marc Maurer (uwog@uwog.net)
  * Copyright (C) 2006 Fridrich Strba (fridrich.strba@bluewin.ch)
  *
@@ -34,7 +34,7 @@
 #include "WP42ContentListener.h"
 
 WP42Parser::WP42Parser(WPXInputStream *input) :
-	WPXParser(input, NULL)
+	WPXParser(input, 0)
 {
 }
 
@@ -65,7 +65,7 @@ void WP42Parser::parseDocument(WPXInputStream *input, WP42Listener *listener)
 
 		if (readVal < (uint8_t)0x20)
 		{
-			WPD_DEBUG_MSG(("Offset: %i, Handling Control Character 0x%2x\n", input->tell(), readVal));			
+			WPD_DEBUG_MSG(("Offset: %i, Handling Control Character 0x%2x\n", (unsigned int)input->tell(), readVal));			
 
 			switch (readVal)
 			{
@@ -91,14 +91,14 @@ void WP42Parser::parseDocument(WPXInputStream *input, WP42Listener *listener)
 		}
 		else if (readVal >= (uint8_t)0x20 && readVal <= (uint8_t)0x7F)
 		{
-			WPD_DEBUG_MSG(("Offset: %i, Handling Ascii Character 0x%2x\n", input->tell(), readVal));			
+			WPD_DEBUG_MSG(("Offset: %i, Handling Ascii Character 0x%2x\n", (unsigned int)input->tell(), readVal));			
 
 			// normal ASCII characters
 			listener->insertCharacter( readVal );
 		}
 		else if (readVal >= (uint8_t)0x80 && readVal <= (uint8_t)0xBF)
 		{
-			WPD_DEBUG_MSG(("Offset: %i, Handling Single Character Function 0x%2x\n", input->tell(), readVal));			
+			WPD_DEBUG_MSG(("Offset: %i, Handling Single Character Function 0x%2x\n", (unsigned int)input->tell(), readVal));			
 
 			// single character function codes
 			switch (readVal)
@@ -148,10 +148,10 @@ void WP42Parser::parseDocument(WPXInputStream *input, WP42Listener *listener)
 					break;
 			}
 		}
-		else if (readVal >= (uint8_t)0xC0 && readVal <= (uint8_t)0xF8)
+		else if (readVal >= (uint8_t)0xC0 && readVal <= (uint8_t)0xFE)
 		{
 			WP42Part *part = WP42Part::constructPart(input, readVal);
-			if (part != NULL)
+			if (part)
 			{
 				part->parse(listener);
 				DELETEP(part);
