@@ -1,7 +1,8 @@
 /* libwpd
  * Copyright (C) 2003 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2003 Marc Maurer (uwog@uwog.net)
- * Copyright (C) 2005-2006 Fridrich Strba (fridrich.strba@bluewin.ch)
+ * Copyright (C) 2005-2007 Fridrich Strba (fridrich.strba@bluewin.ch)
+ * Copyright (C) 2007 Novell, Inc. (http://www.novell.com)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -33,7 +34,6 @@
 #include "WPXString.h"
 #include "libwpd_internal.h"
 #include "WPXHLListenerImpl.h"
-#// include "WPXListener.h"
 #include <vector>
 
 typedef struct _WP5ContentParsingState WP5ContentParsingState;
@@ -55,8 +55,10 @@ public:
 
 	void startDocument() { WPXContentListener::startDocument(); };
 	void setFont(const WPXString &fontName, const float fontSize);
+	void setTabs(const std::vector<WPXTabStop> &tabStops, const uint16_t tabOffset);
 	void insertCharacter(const uint16_t character);
-	void insertTab(const uint8_t tabType, float tabPosition);
+	void insertTab(const uint8_t tabType, const float tabPosition);
+	void insertIndent(const uint8_t indentType, const float indentPosition);
 	void insertEOL();
 	void insertBreak(const uint8_t breakType) { WPXContentListener::insertBreak(breakType); };
 	void lineSpacingChange(const float lineSpacing) { WPXContentListener::lineSpacingChange(lineSpacing); };
@@ -94,6 +96,8 @@ protected:
 	void _changeList() {};
 
 private:
+	WP5ContentListener(const WP5ContentListener&);
+	WP5ContentListener& operator=(const WP5ContentListener&);
 	WP5ContentParsingState *m_parseState;
 	std::vector<WP5SubDocument *> &m_subDocuments;
 	float m_defaultFontSize;
