@@ -37,6 +37,8 @@ public:
 	void parse(WP5Listener *listener);
 
 private:
+	WP5HeaderFooterGroup(const WP5HeaderFooterGroup&);
+	WP5HeaderFooterGroup& operator=(const WP5HeaderFooterGroup&);
 	uint8_t m_occurenceBits;
 	WP5SubDocument *m_subDocument;
 };
