@@ -1,6 +1,6 @@
 /* libwpd
- * Copyright (C) 2003 William Lachance (william.lachance@sympatico.ca)
- * Copyright (C) 2003 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2003 William Lachance (wrlach@gmail.com)
+ * Copyright (C) 2003 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  * For further information visit http://libwpd.sourceforge.net
  */
@@ -24,10 +24,9 @@
  */
 
 #include "WP5UnsupportedFixedLengthGroup.h"
-#include "WP5LLListener.h"
 #include "libwpd_internal.h"
 
-WP5UnsupportedFixedLengthGroup::WP5UnsupportedFixedLengthGroup(GsfInput *input, guint8 groupID)
+WP5UnsupportedFixedLengthGroup::WP5UnsupportedFixedLengthGroup(WPXInputStream *input, uint8_t groupID)
 	: WP5FixedLengthGroup(groupID)
 {
 	_read(input);
