@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2003 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2003 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2003 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
@@ -35,7 +35,7 @@ class WP5UnsupportedVariableLengthGroup : public WP5VariableLengthGroup
 {
 public:
 	WP5UnsupportedVariableLengthGroup(WPXInputStream *input);
-	void parse(WP5Listener *listener) {}
+	void parse(WP5Listener * /* listener */) {}
 };
 
 #endif /* WP5UNSUPPORTEDVARIABLELENGTHGROUP_H */
