@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2003 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2003 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2003 Marc Maurer (uwog@uwog.net)
  * Copyright (C) 2006 Fridrich Strba (fridrich.strba@bluewin.ch)
  *  
@@ -48,7 +48,7 @@ public:
 
 protected:
 	void _read(WPXInputStream *input);
- 	virtual void _readContents(WPXInputStream *input) {} // we don't always need more information than that provided generically
+ 	virtual void _readContents(WPXInputStream * /* input */) {} // we don't always need more information than that provided generically
 
 	const uint8_t getSubGroup() const { return m_subGroup; }
 	const uint16_t getSize() const { return m_size; }
