@@ -39,6 +39,7 @@
 
 #include <assert.h>
 #include <algorithm>
+#include <time.h>
 
 #define WP6_DEFAULT_FONT_SIZE 12.0f
 #define WP6_DEFAULT_FONT_NAME "Times New Roman"
@@ -101,6 +102,12 @@ void WP6OutlineDefinition::_updateNumberingMethods(const WP6OutlineLocation /* o
 }
 
 _WP6ContentParsingState::_WP6ContentParsingState(WPXTableList tableList, int nextTableIndice) :
+	m_bodyText(),
+	m_textBeforeNumber(),
+	m_textBeforeDisplayReference(),
+	m_numberText(),
+	m_textAfterDisplayReference(),
+	m_textAfterNumber(),
 	m_paragraphMarginBottomRelative(1.0f),
 	m_paragraphMarginBottomAbsolute(0.0f),
 
@@ -113,9 +120,13 @@ _WP6ContentParsingState::_WP6ContentParsingState(WPXTableList tableList, int nex
 	m_currentTable(0),
 
 	m_nextTableIndice(nextTableIndice),
+	
+	m_listLevelStack(),
+	m_listTypeStack(),
 
 	m_currentOutlineHash(0),
 	m_oldListLevel(0),
+	m_styleStateSequence(),
 	m_putativeListElementHasParagraphNumber(false),
 	m_putativeListElementHasDisplayReferenceNumber(false),
 	
@@ -123,7 +134,10 @@ _WP6ContentParsingState::_WP6ContentParsingState(WPXTableList tableList, int nex
 	m_numNestedNotes(0),
 
 	m_leaderCharacter('.'),
-	m_leaderNumSpaces(0)
+	m_leaderNumSpaces(0),
+
+	m_tempTabStops(),
+	m_tempUsePreWP9LeaderMethod()
 
 {
 }
@@ -136,7 +150,8 @@ _WP6ContentParsingState::~_WP6ContentParsingState()
 WP6ContentListener::WP6ContentListener(std::list<WPXPageSpan> &pageList, WPXTableList tableList, WPXHLListenerImpl *listenerImpl) :
 	WP6Listener(),
 	WPXContentListener(pageList, listenerImpl),
-	m_parseState(new WP6ContentParsingState(tableList))
+	m_parseState(new WP6ContentParsingState(tableList)),
+	m_outlineDefineHash()
 {
 }
 
@@ -150,12 +165,80 @@ WP6ContentListener::~WP6ContentListener()
 	delete m_parseState;
 }
 
+void WP6ContentListener::setDate(const uint16_t type, const uint16_t year, 
+				 const uint8_t month, const uint8_t day, 
+				 const uint8_t hour, const uint8_t minute, 
+				 const uint8_t second, const uint8_t dayOfWeek, 
+				 const uint8_t /* timeZone */, const uint8_t /* unused */)
+{
+        #define DATEBUFLEN 100  // length of buffer allocated for strftime()
+        #define WPMONDAYOFFSET  1       // WP week starts Monday, tm_wday on Sunday
+        #define TMMONTHOFFSET   1       // tm_mon is [0,11], i.e. months since January
+        #define TMYEAROFFSET    1900    // tm_year is days since 1900
+        #define DAYSINWEEK      7       // I hate magic numbers buried in code :-)
+        char    dateBuffer[DATEBUFLEN];    // points to buffer allocated for strftime()
+        int     retVal;         // return value from strftime()
+        struct  tm      m_tm;   // passed to strftime();
+        const char    *dateFormat = "%Y-%m-%dT%H:%M:%S";
+			// This date format is ALMOST the "Complete date plus hours, minutes and 
+			// seconds" format described at http://www.w3.org/TR/NOTE-datetime, i.e.
+			// YYYY-MM-DDThh:mm:ssTZD (eg 1997-07-16T19:20:30+01:00), but without TZD.
+			// WordPerfect does not use the timeZone field, so we don't know the offset
+        WPXString dateStr;      // filled in and passed to m_metaData.insert()
+
+        m_tm.tm_sec       = second;
+        m_tm.tm_min       = minute;
+        m_tm.tm_hour      = hour;
+        m_tm.tm_mday      = day;
+        m_tm.tm_mon       = month - TMMONTHOFFSET;
+        m_tm.tm_year      = year - TMYEAROFFSET;
+        m_tm.tm_wday      = (dayOfWeek + WPMONDAYOFFSET) % DAYSINWEEK;
+        m_tm.tm_yday      = 0;
+        m_tm.tm_isdst     = -1;
+
+        retVal = strftime(dateBuffer, DATEBUFLEN, dateFormat, &m_tm);
+        if (retVal <= 0) {
+                // error - date wouldn't fit in buffer
+                dateStr.sprintf("ERROR: %d character buffer too short for date", DATEBUFLEN);
+        }
+        else {
+                dateStr.sprintf("%s", dateBuffer);
+        }
+
+	switch (type)
+	{
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_CREATION_DATE):
+			m_metaData.insert("meta:creation-date", dateStr);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DATE_COMPLETED):
+			// output two forms until we decide which to use
+			m_metaData.insert("libwpd:completed-date", dateStr);
+			m_metaData.insert("dcterms:available", dateStr);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_RECORDED_DATE):
+			m_metaData.insert("libwpd:recorded-date", dateStr);
+			break;
+		// NOTE: Revision Date is not set in WP Document Summary
+		// and sometimes contains non-zero data, so it's confusing.
+		// WordPerfect actually uses the file modification time
+		// to display Revision Date in Properties.
+		//case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_REVISION_DATE):
+		//	m_metaData.insert("libwpd:revision-date", dateStr);
+		//	break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_VERSION_DATE):
+			// output two forms until we decide which to use
+			m_metaData.insert("libwpd:version-date", dateStr);
+			m_metaData.insert("dcterms:issued", dateStr);
+			break;
+	}
+}
+
 void WP6ContentListener::setExtendedInformation(const uint16_t type, const WPXString &data)
 {
 	switch (type)
 	{
 		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_AUTHOR):
-			m_metaData.insert("dc:creator", data);
+			m_metaData.insert("meta:initial-creator", data);
 			break;
 		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_SUBJECT):
 			m_metaData.insert("dc:subject", data);
@@ -167,13 +250,13 @@ void WP6ContentListener::setExtendedInformation(const uint16_t type, const WPXSt
 			m_metaData.insert("dc:type", data);
 			break;
 		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_KEYWORDS):
-			m_metaData.insert("libwpd:keywords", data);
+			m_metaData.insert("meta:keyword", data);
 			break;
 		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_LANGUAGE):
 			m_metaData.insert("dc:language", data);
 			break;
 		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_ABSTRACT):
-			m_metaData.insert("libwpd:abstract", data);
+			m_metaData.insert("dc:description", data);
 			break;
 		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DESCRIPTIVE_NAME):
 			m_metaData.insert("libwpd:descriptive-name", data);
@@ -181,6 +264,119 @@ void WP6ContentListener::setExtendedInformation(const uint16_t type, const WPXSt
 		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DESCRIPTIVE_TYPE):
 			m_metaData.insert("libwpd:descriptive-type", data);
 			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_ACCOUNT):
+			m_metaData.insert("libwpd:account", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_ADDRESS):
+			m_metaData.insert("libwpd:address", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_ATTACHMENTS):
+			m_metaData.insert("libwpd:attachments", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_AUTHORIZATION):
+			m_metaData.insert("libwpd:authorization", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_BILL_TO):
+			m_metaData.insert("libwpd:bill-to", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_BLIND_COPY):
+			m_metaData.insert("libwpd:blind-copy", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_CARBON_COPY):
+			m_metaData.insert("libwpd:carbon-copy", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_CHECKED_BY):
+			m_metaData.insert("libwpd:checked-by", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_CLIENT):
+			m_metaData.insert("libwpd:client", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_COMMENTS):
+			m_metaData.insert("libwpd:comments", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DEPARTMENT):
+			m_metaData.insert("libwpd:department", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DESTINATION):
+			m_metaData.insert("libwpd:destination", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DISPOSITION):
+			m_metaData.insert("libwpd:disposition", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DIVISION):
+			m_metaData.insert("libwpd:division", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DOCUMENT_NUMBER):
+			m_metaData.insert("libwpd:document-number", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_EDITOR):
+			m_metaData.insert("libwpd:editor", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_FORWARD_TO):
+			m_metaData.insert("libwpd:forward-to", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_GROUP):
+			m_metaData.insert("libwpd:group", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_MAIL_STOP):
+			m_metaData.insert("libwpd:mail-stop", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_MATTER):
+			m_metaData.insert("libwpd:matter", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_OFFICE):
+			m_metaData.insert("libwpd:office", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_OWNER):
+			m_metaData.insert("libwpd:owner", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_PROJECT):
+			m_metaData.insert("libwpd:project", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_PURPOSE):
+			m_metaData.insert("libwpd:purpose", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_RECEIVED_FROM):
+			m_metaData.insert("libwpd:received-from", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_RECORDED_BY):
+			m_metaData.insert("libwpd:recorded-by", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_REFERENCE):
+			m_metaData.insert("libwpd:reference", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_REVISION_NOTES):
+			m_metaData.insert("libwpd:revision-notes", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_REVISION_NUMBER):
+			m_metaData.insert("libwpd:revision-number", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_SECTION):
+			m_metaData.insert("libwpd:section", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_SECURITY):
+			m_metaData.insert("libwpd:security", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_SOURCE):
+			m_metaData.insert("dc:source", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_STATUS):
+			m_metaData.insert("libwpd:status", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_TELEPHONE_NUMBER):
+			m_metaData.insert("libwpd:telephone-number", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_TYPIST):
+			m_metaData.insert("dc:creator", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_VERSION_NOTES):
+			m_metaData.insert("libwpd:version-notes", data);
+			break;
+		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_VERSION_NUMBER):
+			m_metaData.insert("libwpd:version-number", data);
+			break;
+		default:
+		        break;
 	}
 }
 
@@ -717,6 +913,8 @@ void WP6ContentListener::columnChange(const WPXTextColumnType /* columnType */,
 {
 	if (!isUndoOn())
 	{
+		_openPageSpan(); // we need to have the right dimension, so open page span here if it is not already opened
+
 		int oldColumnNum = m_ps->m_numColumns;
 
 		// In WP, the last column ends with a hard column break code.
@@ -773,13 +971,16 @@ void WP6ContentListener::columnChange(const WPXTextColumnType /* columnType */,
 		m_ps->m_numColumns = numColumns;
 		m_ps->m_textColumns = tmpColumnDefinition;
 		m_ps->m_isTextColumnWithoutParagraph = true;
+
 		if ((m_ps->m_numColumns > 1 && oldColumnNum <= 1) || (m_ps->m_numColumns <= 1 && oldColumnNum > 1))
 		{
 			m_ps->m_paragraphMarginLeft -= m_ps->m_leftMarginByPageMarginChange;
+			m_ps->m_listReferencePosition -= m_ps->m_leftMarginByPageMarginChange;
 			m_ps->m_paragraphMarginRight -= m_ps->m_rightMarginByPageMarginChange;
 			std::swap(m_ps->m_leftMarginByPageMarginChange, m_ps->m_sectionMarginLeft);
 			std::swap(m_ps->m_rightMarginByPageMarginChange, m_ps->m_sectionMarginRight);
 			m_ps->m_paragraphMarginLeft += m_ps->m_leftMarginByPageMarginChange;
+			m_ps->m_listReferencePosition += m_ps->m_leftMarginByPageMarginChange;
 			m_ps->m_paragraphMarginRight += m_ps->m_rightMarginByPageMarginChange;
 		}
 	}
@@ -970,7 +1171,13 @@ void WP6ContentListener::noteOn(const uint16_t textPID)
 			return;
 		}
 
-		_closeSpan();
+		if (!m_ps->m_isParagraphOpened)
+			_openParagraph();
+		else
+		{
+			_flushText();
+			_closeSpan();
+		}
 		m_parseState->m_styleStateSequence.setCurrentState(DOCUMENT_NOTE);
 		// save a reference to the text PID, we want to parse
 		// the packet after we're through with the footnote ref.
@@ -1054,11 +1261,13 @@ void WP6ContentListener::defineTable(const uint8_t position, const uint16_t left
 		m_ps->m_tableDefinition.m_leftOffset = (float)((double)leftOffset / (double)WPX_NUM_WPUS_PER_INCH) - m_ps->m_paragraphMarginLeft;
 
 		// remove all the old column information
-		m_ps->m_tableDefinition.columns.clear();
-		m_ps->m_tableDefinition.columnsProperties.clear();
+		m_ps->m_tableDefinition.m_columns.clear();
+		m_ps->m_tableDefinition.m_columnsProperties.clear();
 
 		// pull a table definition off of our stack
 		m_parseState->m_currentTable = m_parseState->m_tableList[m_parseState->m_nextTableIndice++];
+		if (!m_parseState->m_currentTable)
+			throw ParseException();
 		m_parseState->m_currentTable->makeBordersConsistent();
 		m_ps->m_numRowsToSkip.clear();
 	}
@@ -1080,9 +1289,9 @@ void WP6ContentListener::addTableColumnDefinition(const uint32_t width, const ui
 		colProp.m_alignment = alignment;
 		
 		// add the new column definition to our table definition
-		m_ps->m_tableDefinition.columns.push_back(colDef);
+		m_ps->m_tableDefinition.m_columns.push_back(colDef);
 		
-		m_ps->m_tableDefinition.columnsProperties.push_back(colProp);
+		m_ps->m_tableDefinition.m_columnsProperties.push_back(colProp);
 
 		// initialize the variable that tells us how many columns to skip
 		m_ps->m_numRowsToSkip.push_back(0);
@@ -1107,7 +1316,12 @@ void WP6ContentListener::startTable()
 			_openSection();
 			m_ps->m_sectionAttributesChanged = false;
 		}
-		_openTable();
+		if (!m_parseState->m_currentTable)
+			throw ParseException();
+		
+		if (!m_parseState->m_currentTable->isEmpty()) // WordPerfect ignores empty tables and inserts an Hard EOL instead
+			_openTable();
+			
 	}
 }
 
@@ -1161,8 +1375,8 @@ void WP6ContentListener::insertCell(const uint8_t colSpan, const uint8_t rowSpan
 		if (useCellAttributes)
 			m_ps->m_cellAttributeBits = cellAttributes;
 		else
-			m_ps->m_cellAttributeBits = m_ps->m_tableDefinition.columnsProperties[m_ps->m_currentTableCol-1].m_attributes;
-		justificationChange(m_ps->m_tableDefinition.columnsProperties[m_ps->m_currentTableCol-1].m_alignment);
+			m_ps->m_cellAttributeBits = m_ps->m_tableDefinition.m_columnsProperties[m_ps->m_currentTableCol-1].m_attributes;
+		justificationChange(m_ps->m_tableDefinition.m_columnsProperties[m_ps->m_currentTableCol-1].m_alignment);
 	}
 }
 
@@ -1200,6 +1414,8 @@ void WP6ContentListener::_handleSubDocument(const WPXSubDocument *subDocument, c
 		_openSpan();
 	
 	// Close the sub-document properly
+	if (m_ps->m_isTableOpened)
+		_closeTable();
 	if (m_ps->m_isParagraphOpened)
 		_closeParagraph();
 	if (m_ps->m_isListElementOpened)
