@@ -35,7 +35,6 @@ WP6EOLGroup::WP6EOLGroup(WPXInputStream *input) :
 	WP6VariableLengthGroup(),
 	m_colSpan(1),
 	m_rowSpan(1),
-	m_boundFromLeft(false),
 	m_boundFromAbove(false),
 	
 	m_useCellAttributes(false),
@@ -70,7 +69,7 @@ WP6EOLGroup::~WP6EOLGroup()
 void WP6EOLGroup::_readContents(WPXInputStream *input)
 {
 	WPD_DEBUG_MSG(("WordPerfect: EOL Group: Reading Embedded Sub-Function Data\n"));
-	unsigned int startPosition = input->tell();
+	long startPosition = input->tell();
 	uint16_t sizeDeletableSubFunctionData = readU16(input);		
 	WPD_DEBUG_MSG(("WordPerfect: EOL Group: Size of Deletable Sub-Function Data: %ld,  Size of Deletable and Non-deletable sub-function data: %ld\n", (long) sizeDeletableSubFunctionData, (long) getSizeNonDeletable()));
 	if ((long)sizeDeletableSubFunctionData > (long)getSizeNonDeletable())
@@ -164,20 +163,11 @@ void WP6EOLGroup::_readContents(WPXInputStream *input)
 			case WP6_EOL_GROUP_CELL_SPANNING_INFORMATION:
 				WPD_DEBUG_MSG(("WordPerfect: EOL Group Embedded Sub-Function: CELL_SPANNING_INFORMATION\n"));
 				numBytesToSkip = WP6_EOL_GROUP_CELL_SPANNING_INFORMATION_SIZE;
-				uint8_t numCellsSpannedHorizontally;
-				uint8_t numCellsSpannedVertically;
-				numCellsSpannedHorizontally = readU8(input);
-				numCellsSpannedVertically = readU8(input);
-				WPD_DEBUG_MSG(("WordPerfect: num cells spanned (h:%ld, v:%ld)\n", 
-						   (long) numCellsSpannedHorizontally, (long) numCellsSpannedVertically));
-				if (numCellsSpannedHorizontally >= 128)
-					m_boundFromLeft = true;
-				else
-					m_colSpan = numCellsSpannedHorizontally;
-				if (numCellsSpannedVertically >= 128)
+				m_colSpan = readU8(input);
+				m_rowSpan = readU8(input);
+				WPD_DEBUG_MSG(("WordPerfect: num cells spanned (h:%ld, v:%ld)\n", (long) m_colSpan, (long) m_rowSpan));
+				if (m_colSpan >= 128)
 					m_boundFromAbove = true;
-				else
-					m_rowSpan = numCellsSpannedVertically;
 				break;
 			case WP6_EOL_GROUP_CELL_FILL_COLORS:
 				WPD_DEBUG_MSG(("WordPerfect: EOL Group Embedded Sub-Function: CELL_FILL_COLORS\n"));
@@ -246,6 +236,8 @@ void WP6EOLGroup::_readContents(WPXInputStream *input)
 				throw FileException();
 		}			
 		
+		if (startPosition2 + numBytesToSkip - 1 - input->tell() < 0)
+			throw FileException();
 		input->seek((startPosition2 + numBytesToSkip - 1 - input->tell()), WPX_SEEK_CUR);
 	}
 }
@@ -306,7 +298,7 @@ void WP6EOLGroup::parse(WP6Listener *listener)
 			break;
 		case WP6_EOL_GROUP_TABLE_CELL: // Table Cell
 			WPD_DEBUG_MSG(("WordPerfect: EOL group: table cell\n"));
-			if (!m_boundFromLeft && !m_boundFromAbove)
+			if (!m_boundFromAbove)
 			{
 				listener->insertCell(m_colSpan, m_rowSpan, m_cellBorders, cellFgColor, cellBgColor,
 					cellBorderColor, m_cellVerticalAlign, m_useCellAttributes, m_cellAttributes);
@@ -324,7 +316,7 @@ void WP6EOLGroup::parse(WP6Listener *listener)
 			
 			listener->insertRow(m_rowHeight, m_isMinimumHeight, m_isHeaderRow);
 			// the cellBorders variable already represent the cell border bits as well
-			if (!m_boundFromLeft && !m_boundFromAbove)
+			if (!m_boundFromAbove)  // if m_boundFromLeft is true here, the documentation is lying because this is the first cell in the row
 			{
 				listener->insertCell(m_colSpan, m_rowSpan, m_cellBorders, cellFgColor, cellBgColor,
 					cellBorderColor, m_cellVerticalAlign, m_useCellAttributes, m_cellAttributes);
