@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2002 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
@@ -37,8 +37,10 @@ public:
 	void parse(WP6Listener *listener);
 
  private:
+ 	WP6EOLGroup(const WP6EOLGroup&);
+ 	WP6EOLGroup& operator=(const WP6EOLGroup&);
 	uint8_t m_colSpan, m_rowSpan;
-	bool m_boundFromLeft, m_boundFromAbove;
+	bool m_boundFromAbove;
 
  	bool m_useCellAttributes, m_useCellJustification;
 	bool m_ignoreInCalculations, m_cellIsLocked;
