@@ -32,6 +32,7 @@
 WP6ExtendedDocumentSummaryPacket::WP6ExtendedDocumentSummaryPacket(WPXInputStream *input, int /* id */, uint32_t dataOffset, uint32_t dataSize) :
 	WP6PrefixDataPacket(input),
 	m_dataSize(dataSize),
+	m_streamData(0),
 	m_stream(0)
 {	
 	if (dataSize > 0)
@@ -42,6 +43,8 @@ WP6ExtendedDocumentSummaryPacket::~WP6ExtendedDocumentSummaryPacket()
 {
 	if (m_stream)
 		DELETEP(m_stream);
+	if (m_streamData)
+		delete [] m_streamData;
 }
 
 void WP6ExtendedDocumentSummaryPacket::_readContents(WPXInputStream *input)
@@ -50,11 +53,11 @@ void WP6ExtendedDocumentSummaryPacket::_readContents(WPXInputStream *input)
 		return;
 	if (m_dataSize > ((std::numeric_limits<uint32_t>::max)() / 2))
 		m_dataSize = ((std::numeric_limits<uint32_t>::max)() / 2);
-	uint8_t *streamData = new uint8_t[m_dataSize];
+	m_streamData = new uint8_t[m_dataSize];
 	for(unsigned i=0; i<(unsigned)m_dataSize; i++)
-		streamData[i] = readU8(input);
+		m_streamData[i] = readU8(input);
 	
-	m_stream = new WPXMemoryInputStream(streamData, (size_t)m_dataSize);
+	m_stream = new WPXMemoryInputStream(m_streamData, (size_t)m_dataSize);
 }
 
 void WP6ExtendedDocumentSummaryPacket::parse(WP6Listener *listener) const
@@ -65,8 +68,15 @@ void WP6ExtendedDocumentSummaryPacket::parse(WP6Listener *listener) const
 
 	for (unsigned i=0; i < (unsigned)m_dataSize && !m_stream->atEOS(); i+=groupLength)
 	{
-		groupLength = readU16(m_stream);
-		if ((groupLength <= 0) || m_stream->atEOS())
+		try
+		{
+			groupLength = readU16(m_stream);
+		}
+		catch (FileException)
+		{
+			return;
+		}
+		if ((groupLength == 0) || m_stream->atEOS())
 			return;
 		uint16_t tagID = readU16(m_stream);
 		if (m_stream->atEOS())
@@ -90,18 +100,29 @@ void WP6ExtendedDocumentSummaryPacket::parse(WP6Listener *listener) const
 		} 
 		
 		if (tagID == WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_CREATION_DATE ||
-		    tagID == WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DATE_COMPLETED)
+		    tagID == WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DATE_COMPLETED ||
+		    tagID == WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_RECORDED_DATE ||
+		    tagID == WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_REVISION_DATE ||
+		    tagID == WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_VERSION_DATE)
 		{
-			uint16_t year = readU16(m_stream);
-			uint8_t month = readU8(m_stream);
-			uint8_t day = readU8(m_stream);
-			uint8_t hour = readU8(m_stream);
-			uint8_t minute = readU8(m_stream);
-			uint8_t second = readU8(m_stream);
-			uint8_t dayOfWeek = readU8(m_stream);
-			uint8_t timeZone = readU8(m_stream);
-			uint8_t unused = readU8(m_stream);
-			listener->setDate(year, month, day, hour, minute, second, dayOfWeek, timeZone, unused);
+			try
+			{
+				uint16_t year = readU16(m_stream);
+				uint8_t month = readU8(m_stream);
+				uint8_t day = readU8(m_stream);
+				uint8_t hour = readU8(m_stream);
+				uint8_t minute = readU8(m_stream);
+				uint8_t second = readU8(m_stream);
+				uint8_t dayOfWeek = readU8(m_stream);
+				uint8_t timeZone = readU8(m_stream);
+				uint8_t unused = readU8(m_stream);
+				if (month > 0 && day > 0 && year >= 1900)
+					listener->setDate(tagID, year, month, day, hour, minute, second, dayOfWeek, timeZone, unused);
+			}
+			catch (FileException)
+			{
+				return;
+			}
 		}
 		else
 		{
@@ -117,8 +138,9 @@ void WP6ExtendedDocumentSummaryPacket::parse(WP6Listener *listener) const
 						      characterSet, &chars);
 				for (int j = 0; j < len; j++)
 					appendUCS4(data, (uint32_t)chars[j]);
-			} 
-			listener->setExtendedInformation(tagID, data);
+			}
+			if (data.len())
+				listener->setExtendedInformation(tagID, data);
 		}
 		m_stream->seek((i+groupLength), WPX_SEEK_SET);
 	}
