@@ -39,8 +39,11 @@ class WP6ExtendedDocumentSummaryPacket : public WP6PrefixDataPacket
 	void _readContents(WPXInputStream *input);
 	void parse(WP6Listener *listener) const;
 
- private:              
+ private:
+ 	WP6ExtendedDocumentSummaryPacket(const WP6ExtendedDocumentSummaryPacket&);            
+ 	WP6ExtendedDocumentSummaryPacket& operator=(const WP6ExtendedDocumentSummaryPacket&);            
 	uint32_t m_dataSize;
+	uint8_t *m_streamData;
 	mutable WPXMemoryInputStream *m_stream;
 	
 };
