@@ -74,6 +74,47 @@ extern int WP6_FIXED_LENGTH_FUNCTION_GROUP_SIZE[16];
 #define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_CREATION_DATE 0x0E
 #define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DATE_COMPLETED 0x0F
 
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_ACCOUNT 2
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_ADDRESS 3
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_ATTACHMENTS 4
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_AUTHORIZATION 6
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_BILL_TO 7
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_BLIND_COPY 8
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_CARBON_COPY 9
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_CHECKED_BY 11
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_CLIENT 12
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_COMMENTS 13
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DEPARTMENT 16
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DESTINATION 19
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DISPOSITION 20
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DIVISION 21
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DOCUMENT_NUMBER 22
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_EDITOR 23
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_FORWARD_TO 24
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_GROUP 25
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_MAIL_STOP 28
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_MATTER 29
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_OFFICE 30
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_OWNER 31
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_PROJECT 32
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_PURPOSE 34
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_RECEIVED_FROM 35
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_RECORDED_BY 36
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_RECORDED_DATE 37
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_REFERENCE 38
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_REVISION_DATE 39	// unused and blank
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_REVISION_NOTES 40
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_REVISION_NUMBER 41
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_SECTION 42
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_SECURITY 43
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_SOURCE 44
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_STATUS 45
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_TELEPHONE_NUMBER 47
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_TYPIST 48
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_VERSION_DATE 49
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_VERSION_NOTES 50
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_VERSION_NUMBER 51
+
 #define WP6_INDEX_HEADER_OUTLINE_STYLE 0x31
 #define WP6_INDEX_HEADER_OUTLINE_STYLE_ARABIC_NUMBERING 0
 #define WP6_INDEX_HEADER_OUTLINE_STYLE_LOWERCASE_NUMBERING 1
