@@ -1,6 +1,6 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
- * Copyright (C) 2002 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
+ * Copyright (C) 2002 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  *
  * For further information visit http://libwpd.sourceforge.net
  */
@@ -23,14 +23,15 @@
  * Corel Corporation or Corel Corporation Limited."
  */
 #include <string.h>
-#include <gsf/gsf-input-memory.h>
 
 #include "WP6FillStylePacket.h"
 #include "WP6Parser.h"
 #include "libwpd_internal.h"
 
-WP6FillStylePacket::WP6FillStylePacket(GsfInput *input, int id, guint32 dataOffset, guint32 dataSize) 
-	: WP6PrefixDataPacket(input)
+WP6FillStylePacket::WP6FillStylePacket(WPXInputStream *input, int /* id */, uint32_t dataOffset, uint32_t dataSize) : 
+	WP6PrefixDataPacket(input),
+	m_fgColor(0xff, 0xff, 0xff),
+	m_bgColor(0xff, 0xff, 0xff)
 {	
 	_read(input, dataOffset, dataSize);
 }
@@ -43,26 +44,26 @@ WP6FillStylePacket::~WP6FillStylePacket()
 
 const int WP6_FILL_STYLE_PACKET_SKIPABLE_DATA_AFTER_PREFIX_PACKETS = 6;
 const int WP6_FILL_STYLE_PACKET_SKIPABLE_DATA_AFTER_FILL_NAME = 3;
-void WP6FillStylePacket::_readContents(GsfInput *input)
+void WP6FillStylePacket::_readContents(WPXInputStream *input)
 {
 	/* skip a whole bunch of useless crap */
-	guint16 numChildPrefixIDs = gsf_le_read_guint16(input);
-	WPD_CHECK_FILE_SEEK_ERROR(gsf_input_seek(input, sizeof(guint16)*numChildPrefixIDs, G_SEEK_CUR));
-	WPD_CHECK_FILE_SEEK_ERROR(gsf_input_seek(input, WP6_FILL_STYLE_PACKET_SKIPABLE_DATA_AFTER_PREFIX_PACKETS, G_SEEK_CUR));
-	gint16 fillNameLength = gsf_le_read_guint16(input);
+	uint16_t numChildPrefixIDs = readU16(input);
+	input->seek(sizeof(uint16_t)*numChildPrefixIDs, WPX_SEEK_CUR);
+	input->seek(WP6_FILL_STYLE_PACKET_SKIPABLE_DATA_AFTER_PREFIX_PACKETS, WPX_SEEK_CUR);
+	int16_t fillNameLength = readU16(input);
 	if (fillNameLength > 0)
-		WPD_CHECK_FILE_SEEK_ERROR(gsf_input_seek(input, fillNameLength*sizeof(guint8), G_SEEK_CUR));
-	WPD_CHECK_FILE_SEEK_ERROR(gsf_input_seek(input, WP6_FILL_STYLE_PACKET_SKIPABLE_DATA_AFTER_FILL_NAME, G_SEEK_CUR));
+		input->seek(fillNameLength*sizeof(uint8_t), WPX_SEEK_CUR);
+	input->seek(WP6_FILL_STYLE_PACKET_SKIPABLE_DATA_AFTER_FILL_NAME, WPX_SEEK_CUR);
 
 	/* now we can finally grab the meat */
-	m_fgColor.m_r = gsf_le_read_guint8(input);
-	m_fgColor.m_g = gsf_le_read_guint8(input);
-	m_fgColor.m_b = gsf_le_read_guint8(input);
-	m_fgColor.m_s = gsf_le_read_guint8(input);
-	m_bgColor.m_r = gsf_le_read_guint8(input);
-	m_bgColor.m_g = gsf_le_read_guint8(input);
-	m_bgColor.m_b = gsf_le_read_guint8(input);
-	m_bgColor.m_s = gsf_le_read_guint8(input);
+	m_fgColor.m_r = readU8(input);
+	m_fgColor.m_g = readU8(input);
+	m_fgColor.m_b = readU8(input);
+	m_fgColor.m_s = readU8(input);
+	m_bgColor.m_r = readU8(input);
+	m_bgColor.m_g = readU8(input);
+	m_bgColor.m_b = readU8(input);
+	m_bgColor.m_s = readU8(input);
 	WPD_DEBUG_MSG(("WordPerfect: Fill Prefix Packet FG Color (%i, %i, %i, %i) BG Color (%i, %i, %i, %i)\n",
 		       m_fgColor.m_r, m_fgColor.m_g, m_fgColor.m_b, m_fgColor.m_s,
 		       m_bgColor.m_r, m_bgColor.m_g, m_bgColor.m_b, m_bgColor.m_s));
