@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2002 Marc Maurer (uwog@uwog.net)
  * Copyright (C) 2006 Fridrich Strba (fridrich.strba@bluewin.ch)
  *  
@@ -24,16 +24,13 @@
  * Corel Corporation or Corel Corporation Limited."
  */
 
-#include "WPXPart.h"
 #include "WP6Part.h"
-#include "WP6Header.h"
 #include "WP6VariableLengthGroup.h"
 #include "WP6FixedLengthGroup.h"
 #include "WP6SingleByteFunction.h"
-#include "libwpd_internal.h"
 
 // constructPart: constructs a parseable low-level representation of part of the document
-// returns the part if it successfully creates the part, returns NULL if it can't
+// returns the part if it successfully creates the part, returns 0 if it can't
 // throws an exception if there is an error
 // precondition: readVal us between 0x80 and 0xFF
 WP6Part * WP6Part::constructPart(WPXInputStream *input, const uint8_t readVal)
@@ -50,23 +47,27 @@ WP6Part * WP6Part::constructPart(WPXInputStream *input, const uint8_t readVal)
 		if (!WP6VariableLengthGroup::isGroupConsistent(input, readVal))
 		{
 			WPD_DEBUG_MSG(("WordPerfect: Consistency Check (variable length) failed; ignoring this byte\n"));
-			return NULL;
+			return 0;
 		}
 		WPD_DEBUG_MSG(("WordPerfect: constructVariableLengthGroup(input, val=0x%.2x)\n", readVal));
 		return WP6VariableLengthGroup::constructVariableLengthGroup(input, readVal);
 	}      
 
-	else if (readVal >= (uint8_t)0xF0)
+	else if (readVal >= (uint8_t)0xF0 && readVal < (uint8_t)0xFF)
 	{
 		if (!WP6FixedLengthGroup::isGroupConsistent(input, readVal))
 		{
 			WPD_DEBUG_MSG(("WordPerfect: Consistency Check (fixed length) failed; ignoring this byte\n"));
-			return NULL;
+			return 0;
 		}
 		WPD_DEBUG_MSG(("WordPerfect: constructFixedLengthGroup(input, val=0x%.2x)\n", readVal));
 		return WP6FixedLengthGroup::constructFixedLengthGroup(input, readVal);
 	}
+#ifdef DEBUG
+	else
+		WPD_DEBUG_MSG(("WordPerfect: invalid group (val=0x%2x)\n", readVal));
+#endif
 
-	WPD_DEBUG_MSG(("WordPerfect: Returning NULL from constructPart\n"));
-	return NULL;
+	WPD_DEBUG_MSG(("WordPerfect: Returning 0 from constructPart\n"));
+	return 0;
 }
