@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2002 Marc Maurer (uwog@uwog.net)
  * Copyright (C) 2006 Fridrich Strba (fridrich.strba@bluewin.ch)
  *  
@@ -27,10 +27,10 @@
 #ifndef WP6PART_H
 #define WP6PART_H
 
-#include "WPXPart.h"
-#include "WPXParser.h"
-#include "WP6Listener.h"
-#include "WP6FileStructure.h"
+#include "libwpd_internal.h"
+
+class WPXInputStream;
+class WP6Listener;
 
 class WP6Part
 {
