@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2002-2003 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
@@ -31,6 +31,8 @@
 #include "libwpd_internal.h"
 
 WP6PrefixData::WP6PrefixData(WPXInputStream *input, const int numPrefixIndices) :
+	m_prefixDataPacketHash(),
+	m_prefixDataPacketTypeHash(),
 	m_defaultInitialFontPID((-1))
 {
 	uint16_t i;
@@ -75,16 +77,12 @@ const WP6PrefixDataPacket * WP6PrefixData::getPrefixDataPacket(const int prefixI
 	if (pos != m_prefixDataPacketHash.end())
 		return static_cast<const WP6PrefixDataPacket *>(pos->second);
 	else
-		return NULL;
+		return 0;
 }
 
-std::pair<MPDP_CIter, MPDP_CIter> * WP6PrefixData::getPrefixDataPacketsOfType(const int type) const
+std::pair<MPDP_CIter, MPDP_CIter> WP6PrefixData::getPrefixDataPacketsOfType(const int type) const
 {
- 	std::pair<MPDP_CIter, MPDP_CIter> *typePair = new std::pair<MPDP_CIter, MPDP_CIter>;
 	std::pair<MPDP_CIter, MPDP_CIter> tempPair = m_prefixDataPacketTypeHash.equal_range(type);
 
- 	typePair->first = tempPair.first;
- 	typePair->second = tempPair.second;
-
- 	return typePair;
+ 	return tempPair;
 }
