@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2002 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
@@ -25,8 +25,8 @@
 
 #ifndef WP6PREFIXDATAPACKET_H
 #define WP6PREFIXDATAPACKET_H
-#include <stdlib.h>
 #include "WPXParser.h"
+#include "libwpd_types.h"
 
 class WP6Listener;
 class WP6PrefixIndice;
@@ -37,14 +37,18 @@ class WP6PrefixDataPacket
 public:
 	WP6PrefixDataPacket(WPXInputStream * input);	
 	virtual ~WP6PrefixDataPacket() {}
-	virtual void parse(WP6Listener *listener) const {}
-	virtual WP6SubDocument * getSubDocument() const { return NULL; }
+	virtual void parse(WP6Listener * /* listener */) const {}
+	virtual WP6SubDocument * getSubDocument() const { return 0; }
+	const uint32_t getDataSize() const { return m_dataSize; }
 
 	static WP6PrefixDataPacket * constructPrefixDataPacket(WPXInputStream * input, WP6PrefixIndice *prefixIndice);
 
 protected:
 	virtual void _readContents(WPXInputStream *input) = 0;
  	void _read(WPXInputStream *input, uint32_t dataOffset, uint32_t dataSize);
+
+private:
+	uint32_t m_dataSize;
 };
 
 #endif /* WP6PREFIXDATAPACKET_H */
