@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2002 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
@@ -24,10 +24,13 @@
  */
 
 #include "WP6UndoGroup.h"
+#include "WP6Listener.h"
 #include "libwpd_internal.h"
 
-WP6UndoGroup::WP6UndoGroup(WPXInputStream *input, uint8_t groupID)
-	: WP6FixedLengthGroup(groupID)
+WP6UndoGroup::WP6UndoGroup(WPXInputStream *input, uint8_t groupID) :
+	WP6FixedLengthGroup(groupID),
+	m_undoType(0),
+	m_undoLevel(0)
 {
 	_read(input);
 }
