@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2002 Marc Maurer (uwog@uwog.net)
  * Copyright (C) 2006 Fridrich Strba (fridrich.strba@bluewin.ch)
  *  
@@ -41,8 +41,12 @@
 
 
 WP6VariableLengthGroup::WP6VariableLengthGroup() :
+	m_subGroup(0),
+	m_size(0),
+	m_flags(0),
 	m_numPrefixIDs(0),
-	m_prefixIDs(NULL)
+	m_prefixIDs(0),
+	m_sizeNonDeletable(0)
 {
 }
 
@@ -88,7 +92,7 @@ bool WP6VariableLengthGroup::isGroupConsistent(WPXInputStream *input, const uint
 
 	try
 	{
-		uint8_t subGroup = readU8(input);
+		input->seek(1, WPX_SEEK_CUR);
 		uint16_t size = readU16(input);
 
 		if (input->seek((startPosition + size - 4 - input->tell()), WPX_SEEK_CUR) || input->atEOS())
@@ -141,7 +145,7 @@ void WP6VariableLengthGroup::_read(WPXInputStream *input)
 	else
 	{
 		m_numPrefixIDs = 0;
-		m_prefixIDs = NULL;
+		m_prefixIDs = 0;
 	}
 		
 	m_sizeNonDeletable = readU16(input);
