@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2003 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2003 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2003-2004 Marc Maurer (uwog@uwog.net)
  *
  * This library is free software; you can redistribute it and/or
@@ -23,13 +23,13 @@
  * Corel Corporation or Corel Corporation Limited."
  */
 
-#include <stdlib.h>
-#include <string.h>
 #include "WPDocument.h"
 #include "WPXHeader.h"
 #include "WPXParser.h"
+#include "WP1Parser.h"
 #include "WP3Parser.h"
 #include "WP42Parser.h"
+#include "WP1Heuristics.h"
 #include "WP42Heuristics.h"
 #include "WP5Parser.h"
 #include "WP6Parser.h"
@@ -63,13 +63,13 @@ WPDConfidence WPDocument::isFileFormatSupported(WPXInputStream *input, bool part
 {
 	WPDConfidence confidence = WPD_CONFIDENCE_NONE;
 
-	WPXHeader *header = NULL;
+	WPXHeader *header = 0;
 
 	WPD_DEBUG_MSG(("WPDocument::isFileFormatSupported()\n"));
 
 	// by-pass the OLE stream (if it exists) and returns the (sub) stream with the
 	// WordPerfect document.
-	WPXInputStream *document = NULL;
+	WPXInputStream *document = 0;
 	bool isDocumentOLE = false;
 
 	// BIG BIG NOTE: very unsafe on partial content!!!
@@ -130,14 +130,19 @@ WPDConfidence WPDocument::isFileFormatSupported(WPXInputStream *input, bool part
 					// unhandled file type
 					confidence = WPD_CONFIDENCE_NONE;
 					break;
-			}						
+			}
+			if (header->getDocumentEncryption())
+				confidence = WPD_CONFIDENCE_NONE; // do not handle password protected documents						
 			DELETEP(header);
 		}
 		else
-			confidence = WP42Heuristics::isWP42FileFormat(input, partialContent);
+			confidence = WP1Heuristics::isWP1FileFormat(input, partialContent);
+			if (confidence != WPD_CONFIDENCE_EXCELLENT)
+				confidence = LIBWPD_MAX(confidence, WP42Heuristics::isWP42FileFormat(input, partialContent));
+			
 
 		// dispose of the reference to the ole input stream, if we allocated one
-		if (document != NULL && isDocumentOLE)
+		if (document && isDocumentOLE)
 			DELETEP(document);
 
 		return confidence;
@@ -147,7 +152,7 @@ WPDConfidence WPDocument::isFileFormatSupported(WPXInputStream *input, bool part
 		WPD_DEBUG_MSG(("File Exception trapped\n"));
 
 		// dispose of the reference to the ole input stream, if we allocated one
-		if (document != NULL && isDocumentOLE)
+		if (document && isDocumentOLE)
 			DELETEP(document);
 
 		return WPD_CONFIDENCE_NONE;
@@ -157,13 +162,11 @@ WPDConfidence WPDocument::isFileFormatSupported(WPXInputStream *input, bool part
 		WPD_DEBUG_MSG(("Unknown Exception trapped\n"));
 
 		// dispose of the reference to the ole input stream, if we allocated one
-		if (document != NULL && isDocumentOLE)
+		if (document && isDocumentOLE)
 			DELETEP(document);
 
 		return WPD_CONFIDENCE_NONE;
 	}
-
-	return WPD_CONFIDENCE_NONE;
 }
 
 /**
@@ -175,12 +178,12 @@ WPXHLListenerImpl class implementation when needed. This is often commonly calle
 */
 WPDResult WPDocument::parse(WPXInputStream *input, WPXHLListenerImpl *listenerImpl)
 {
-	WPXParser *parser = NULL;
+	WPXParser *parser = 0;
 
 	// by-pass the OLE stream (if it exists) and returns the (sub) stream with the
 	// WordPerfect document.
 
-	WPXInputStream *document = NULL;
+	WPXInputStream *document = 0;
 	bool isDocumentOLE = false;
 
 	WPD_DEBUG_MSG(("WPDocument::parse()\n"));
@@ -201,7 +204,7 @@ WPDResult WPDocument::parse(WPXInputStream *input, WPXHLListenerImpl *listenerIm
 	{
 		WPXHeader *header = WPXHeader::constructHeader(document);
 
-		if (header)
+		if (header && !header->getDocumentEncryption())
 		{
 			switch (header->getFileType())
 			{
@@ -248,23 +251,35 @@ WPDResult WPDocument::parse(WPXInputStream *input, WPXHLListenerImpl *listenerIm
 
 			DELETEP(parser); // deletes the header as well
 		}
+		else if (header && header->getDocumentEncryption())
+		{
+			DELETEP(header);
+			throw UnsupportedEncryptionException();
+		}
 		else
 		{
 			// WP file formats prior to version 5.x do not contain a generic
 			// header which can be used to determine which parser to instanciate.
 			// Use heuristics to determine with some certainty if we are dealing with
-			// a file in the WP4.2 format.
-			int confidence = WP42Heuristics::isWP42FileFormat(document, false /* FIXME: allow for partial content */);
-
-			if (confidence != WPD_CONFIDENCE_NONE)
+			// a file in the WP4.2 format or WP Mac 1.x format.
+			if (WP1Heuristics::isWP1FileFormat(document, false) != WPD_CONFIDENCE_NONE)
+			{
+				WPD_DEBUG_MSG(("WordPerfect: Mostly likely the file format is WP Mac 1.x.\n\n"));
+				WPD_DEBUG_MSG(("WordPerfect: Using the WP Mac 1.x parser.\n\n"));
+				parser = new WP1Parser(document);
+				parser->parse(listenerImpl);
+				DELETEP(parser);
+			}
+			else if (WP42Heuristics::isWP42FileFormat(document, false) != WPD_CONFIDENCE_NONE)
 			{
 				WPD_DEBUG_MSG(("WordPerfect: Mostly likely the file format is WP4.2.\n\n"));
 				WPD_DEBUG_MSG(("WordPerfect: Using the WP4.2 parser.\n\n"));
-				WP42Parser *parser = new WP42Parser(document);
+				parser = new WP42Parser(document);
 				parser->parse(listenerImpl);
 				DELETEP(parser);
 			}
 			else
+
 				error = WPD_FILE_ACCESS_ERROR;
 		}
 	}
@@ -291,7 +306,7 @@ WPDResult WPDocument::parse(WPXInputStream *input, WPXHLListenerImpl *listenerIm
 	}
 
 	DELETEP(parser);
-	if (document != NULL && isDocumentOLE)
+	if (document && isDocumentOLE)
 		DELETEP(document);
 
 	return error;
