@@ -26,21 +26,14 @@
 #include "WPXPageSpan.h"
 #include "libwpd_internal.h"
 #include "WPXProperty.h"
-#ifdef _MSC_VER
-#include <minmax.h>
-#define LIBWPD_MIN min
-#define LIBWPD_MAX max
-#else
-#define LIBWPD_MIN std::min
-#define LIBWPD_MAX std::max
-#endif
+#include <limits>
 
 _WPXContentParsingState::_WPXContentParsingState() :
 	m_textAttributeBits(0),
 	m_fontSize(12.0f/*WP6_DEFAULT_FONT_SIZE*/), // FIXME ME!!!!!!!!!!!!!!!!!!! HELP WP6_DEFAULT_FONT_SIZE
 	m_fontName(new WPXString(/*WP6_DEFAULT_FONT_NAME*/"Times New Roman")), // EN PAS DEFAULT FONT AAN VOOR WP5/6/etc
 	m_fontColor(new RGBSColor(0x00,0x00,0x00,0x64)), //Set default to black. Maybe once it will change, but for the while...
-	m_highlightColor(NULL),
+	m_highlightColor(0),
 
 	m_isParagraphColumnBreak(false),
 	m_isParagraphPageBreak(false),
@@ -57,23 +50,27 @@ _WPXContentParsingState::_WPXContentParsingState() :
 	m_isSpanOpened(false),
 	m_isParagraphOpened(false),
 	m_isListElementOpened(false),
-
+	
+	m_numRowsToSkip(),
+	m_tableDefinition(),
 	m_currentTableCol(0),
 	m_currentTableRow(0),
 	m_currentTableCellNumberInRow(0),
 	m_isTableOpened(false),
 	m_isTableRowOpened(false),
+	m_isTableColumnOpened(false),
 	m_isTableCellOpened(false),
 	m_wasHeaderRow(false),
 	m_isCellWithoutParagraph(false),
 	m_cellAttributeBits(0x00000000),
 	m_paragraphJustificationBeforeTable(WPX_PARAGRAPH_JUSTIFICATION_LEFT),
-	m_paragraphJustificationBeforeColumns(WPX_PARAGRAPH_JUSTIFICATION_LEFT),
-
+	
+	m_nextPageSpanIter(),
 	m_numPagesRemainingInSpan(0),
 
 	m_sectionAttributesChanged(false),
 	m_numColumns(1),
+	m_textColumns(),
 	m_isTextColumnWithoutParagraph(false),
 
 	m_pageFormLength(11.0f),
@@ -105,7 +102,11 @@ _WPXContentParsingState::_WPXContentParsingState() :
 	m_currentListLevel(0),
 
 	m_alignmentCharacter('.'),
+	m_tabStops(),
 	m_isTabPositionRelative(false),
+	
+	m_subDocuments(),
+	
 	m_inSubDocument(false),
 	m_isNote(false)
 {
@@ -121,7 +122,8 @@ _WPXContentParsingState::~_WPXContentParsingState()
 WPXContentListener::WPXContentListener(std::list<WPXPageSpan> &pageList, WPXHLListenerImpl *listenerImpl) :
 	WPXListener(pageList),
 	m_ps(new WPXContentParsingState),
-	m_listenerImpl(listenerImpl)
+	m_listenerImpl(listenerImpl),
+	m_metaData()
 {
 	m_ps->m_nextPageSpanIter = pageList.begin();
 }
@@ -297,9 +299,10 @@ void WPXContentListener::_openPageSpan()
 	std::vector<WPXHeaderFooter> headerFooterList = currentPage.getHeaderFooterList();
 	for (std::vector<WPXHeaderFooter>::iterator iter = headerFooterList.begin(); iter != headerFooterList.end(); iter++)
 	{
-		if (!currentPage.getHeaderFooterSuppression((*iter).getInternalType()))
+		if (((*iter).getOccurence() != NEVER) && ((*iter).getInternalType() != DUMMY_INTERNAL_HEADER_FOOTER)
+			&& !currentPage.getHeaderFooterSuppression((*iter).getInternalType()))
 		{
-			WPXPropertyList propList;
+			propList.clear();
 			switch ((*iter).getOccurence())
 			{
 			case ODD:
@@ -311,6 +314,9 @@ void WPXContentListener::_openPageSpan()
 			case ALL:
 				propList.insert("libwpd:occurence", "all");
 				break;
+			case NEVER:
+			default:
+				break;
 			}
 
 			if ((*iter).getType() == HEADER)
@@ -318,7 +324,9 @@ void WPXContentListener::_openPageSpan()
 			else
 				m_listenerImpl->openFooter(propList); 
 
+			WPD_DEBUG_MSG(("Header Footer Element: Starting to parse the subDocument\n"));
 			handleSubDocument((*iter).getSubDocument(), true, (*iter).getTableList(), 0);
+			WPD_DEBUG_MSG(("Header Footer Element: End of the subDocument parsing\n"));
 			if ((*iter).getType() == HEADER)
 				m_listenerImpl->closeHeader();
 			else
@@ -470,15 +478,15 @@ void WPXContentListener::_appendParagraphProperties(WPXPropertyList &propList, c
 	propList.insert("fo:margin-top", m_ps->m_paragraphMarginTop);
 	propList.insert("fo:margin-bottom", m_ps->m_paragraphMarginBottom);
 	propList.insert("fo:line-height", m_ps->m_paragraphLineSpacing, PERCENT);
-	if (m_ps->m_isParagraphColumnBreak)
-		propList.insert("fo:break-before", "column");
-	else if (m_ps->m_isParagraphPageBreak)
+	if (m_ps->m_isParagraphPageBreak && !m_ps->m_inSubDocument) // no hard page-breaks in subdocuments
 		propList.insert("fo:break-before", "page");
+	else if (m_ps->m_isParagraphColumnBreak)
+		propList.insert("fo:break-before", "column");
 }
 
 void WPXContentListener::_getTabStops(WPXPropertyListVector &tabStops)
 {
-	for (int i=0; i<m_ps->m_tabStops.size(); i++)
+	for (int i=0; i<(int)m_ps->m_tabStops.size(); i++)
 	{
 		WPXPropertyList tmpTabStop;
 
@@ -513,6 +521,8 @@ void WPXContentListener::_getTabStops(WPXPropertyListVector &tabStops)
 			position -= m_ps->m_leftMarginByTabs;
 		else
 			position -= m_ps->m_paragraphMarginLeft + m_ps->m_sectionMarginLeft + m_ps->m_pageMarginLeft;
+		if (position < 0.00005f && position > -0.00005f)
+			position = 0.0f;
 		tmpTabStop.insert("style:position", position);
 		
 
@@ -585,10 +595,10 @@ void WPXContentListener::_openSpan()
 
 	if (!m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
 		_changeList();
-		if (m_ps->m_currentListLevel == 0)
-			_openParagraph();
-		else
-			_openListElement();
+	if (m_ps->m_currentListLevel == 0)
+		_openParagraph();
+	else
+		_openListElement();
 	
 	// The behaviour of WP6+ is following: if an attribute bit is set in the cell attributes, we cannot
 	// unset it; if it is set, we can set or unset it
@@ -624,13 +634,15 @@ void WPXContentListener::_openSpan()
 
 	WPXPropertyList propList;
  	if (attributeBits & WPX_SUPERSCRIPT_BIT) {
-		WPXString sSuperScript;
-		sSuperScript.sprintf("super %f%%", WPX_DEFAULT_SUPER_SUB_SCRIPT);
+		WPXString sSuperScript("super ");
+		sSuperScript.append(doubleToString(WPX_DEFAULT_SUPER_SUB_SCRIPT));
+		sSuperScript.append("%");
 		propList.insert("style:text-position", sSuperScript);
 	}
  	else if (attributeBits & WPX_SUBSCRIPT_BIT) {
-		WPXString sSubScript;
-		sSubScript.sprintf("sub %f%%", WPX_DEFAULT_SUPER_SUB_SCRIPT);
+		WPXString sSubScript("sub ");
+		sSubScript.append(doubleToString(WPX_DEFAULT_SUPER_SUB_SCRIPT));
+		sSubScript.append("%");
 		propList.insert("style:text-position", sSubScript);
 	}
 	if (attributeBits & WPX_ITALICS_BIT)
@@ -716,7 +728,7 @@ void WPXContentListener::_openTable()
 	}
 
 	// cater for the possibility to have the column/page break just before the table
-	if (m_ps->m_isParagraphPageBreak)
+	if (m_ps->m_isParagraphPageBreak && !m_ps->m_inSubDocument)  // No hard page-breaks in sub-documents
 		propList.insert("fo:break-before", "page");
 	else if (m_ps->m_isParagraphColumnBreak)
 		propList.insert("fo:break-before", "column");
@@ -726,7 +738,7 @@ void WPXContentListener::_openTable()
  	float tableWidth = 0.0f;
 	WPXPropertyListVector columns;
  	typedef std::vector<WPXColumnDefinition>::const_iterator CDVIter;
- 	for (CDVIter iter = m_ps->m_tableDefinition.columns.begin(); iter != m_ps->m_tableDefinition.columns.end(); iter++)
+ 	for (CDVIter iter = m_ps->m_tableDefinition.m_columns.begin(); iter != m_ps->m_tableDefinition.m_columns.end(); iter++)
  	{
 		WPXPropertyList column;
 		// The "style:rel-width" is expressed in twips (1440 twips per inch) and includes the left and right Gutter
@@ -809,14 +821,16 @@ void WPXContentListener::_closeTableRow()
 {
 	if (m_ps->m_isTableRowOpened)
 	{
-		while (m_ps->m_currentTableCol < m_ps->m_numRowsToSkip.size())
+		if (m_ps->m_currentTableCol < 0)
+			throw ParseException();
+		while ((unsigned long)m_ps->m_currentTableCol < (unsigned long)m_ps->m_numRowsToSkip.size())
 		{
 			if (!m_ps->m_numRowsToSkip[m_ps->m_currentTableCol]) // This case should not happen, but does :-(
 			{
 				// m_ps->m_currentTableCol++;
 				// Fill the table row untill the end with empty cells
 				RGBSColor tmpCellBorderColor(0x00, 0x00, 0x00, 0x64);
-				_openTableCell(1, 1, 0xFF, NULL, NULL, &tmpCellBorderColor, TOP);
+				_openTableCell(1, 1, 0xFF, 0, 0, &tmpCellBorderColor, TOP);
 				_closeTableCell();
 			}
 			else
@@ -853,11 +867,14 @@ static void addBorderProps(const char *border, bool borderOn, const WPXString &b
 	borderStyle.sprintf("fo:border-%s", border);
 	WPXString props;
 	if (borderOn)
-		props.sprintf("%finch solid %s", WPX_DEFAULT_TABLE_BORDER_WIDTH, borderColor.cstr());
+	{
+	  props.append(doubleToString(WPX_DEFAULT_TABLE_BORDER_WIDTH));
+	  props.append("inch solid ");
+	  props.append(borderColor);
+	}
 	else
 		props.sprintf("0.0inch");
 	propList.insert(borderStyle.cstr(), props);
-	WPXString borderOff;
 }
 
 void WPXContentListener::_openTableCell(const uint8_t colSpan, const uint8_t rowSpan, const uint8_t borderBits,  
@@ -868,7 +885,11 @@ void WPXContentListener::_openTableCell(const uint8_t colSpan, const uint8_t row
 	if (m_ps->m_isTableCellOpened)
 		_closeTableCell();
 
-	while (m_ps->m_currentTableCol < m_ps->m_numRowsToSkip.size() && m_ps->m_numRowsToSkip[m_ps->m_currentTableCol])
+	if (m_ps->m_currentTableCol < 0)
+		throw ParseException();
+
+	while ((unsigned long)m_ps->m_currentTableCol < (unsigned long)m_ps->m_numRowsToSkip.size() &&
+		m_ps->m_numRowsToSkip[m_ps->m_currentTableCol])
 	{
 		m_ps->m_numRowsToSkip[m_ps->m_currentTableCol]--;
 		m_ps->m_currentTableCol++;
@@ -908,7 +929,10 @@ void WPXContentListener::_openTableCell(const uint8_t colSpan, const uint8_t row
 	m_ps->m_isTableCellOpened = true;
 	m_ps->m_isCellWithoutParagraph = true;
 
-	while ((m_ps->m_currentTableCol < m_ps->m_numRowsToSkip.size()) && (tmpColSpan > 0))
+	if (m_ps->m_currentTableCol < 0)
+		throw ParseException();
+
+	while ((unsigned long)m_ps->m_currentTableCol < (unsigned long)m_ps->m_numRowsToSkip.size() &&(tmpColSpan > 0))
 	{
 		if (m_ps->m_numRowsToSkip[m_ps->m_currentTableCol]) // This case should not happen, but it happens in real-life documents :-(
 		{
@@ -1075,6 +1099,42 @@ void WPXContentListener::justificationChange(const uint8_t justification)
 	}
 }
 
+const float WPXContentListener::_getNextTabStop() const
+{
+	for (std::vector<WPXTabStop>::const_iterator iter = m_ps->m_tabStops.begin(); iter != (m_ps->m_tabStops.end() - 1); iter++)
+	{
+		if (iter->m_position
+			- (m_ps->m_isTabPositionRelative ? 0.0f : (m_ps->m_pageMarginLeft + m_ps->m_sectionMarginLeft + m_ps->m_leftMarginByParagraphMarginChange))
+			== (m_ps->m_leftMarginByTabs + m_ps->m_textIndentByTabs + m_ps->m_textIndentByParagraphIndentChange))
+			return (iter+1)->m_position
+				- (m_ps->m_isTabPositionRelative ? 0.0f : (m_ps->m_pageMarginLeft + m_ps->m_sectionMarginLeft + m_ps->m_leftMarginByParagraphMarginChange));
+		if (iter->m_position
+			- (m_ps->m_isTabPositionRelative ? 0.0f : (m_ps->m_pageMarginLeft + m_ps->m_sectionMarginLeft + m_ps->m_leftMarginByParagraphMarginChange))
+			> (m_ps->m_leftMarginByTabs + m_ps->m_textIndentByTabs + m_ps->m_textIndentByParagraphIndentChange))
+			return iter->m_position
+				- (m_ps->m_isTabPositionRelative ? 0.0f : (m_ps->m_pageMarginLeft + m_ps->m_sectionMarginLeft + m_ps->m_leftMarginByParagraphMarginChange));
+	}
+	return (std::numeric_limits<float>::min)();
+}
+
+const float WPXContentListener::_getPreviousTabStop() const
+{
+	for (std::vector<WPXTabStop>::reverse_iterator riter = m_ps->m_tabStops.rbegin(); riter != (m_ps->m_tabStops.rend() - 1); riter++)
+	{
+		if (riter->m_position
+			- (m_ps->m_isTabPositionRelative ? 0.0f : (m_ps->m_pageMarginLeft + m_ps->m_sectionMarginLeft + m_ps->m_leftMarginByParagraphMarginChange))
+			== (m_ps->m_leftMarginByTabs + m_ps->m_textIndentByTabs + m_ps->m_textIndentByParagraphIndentChange))
+			return (riter+1)->m_position
+				- (m_ps->m_isTabPositionRelative ? 0.0f : (m_ps->m_pageMarginLeft + m_ps->m_sectionMarginLeft + m_ps->m_leftMarginByParagraphMarginChange));
+		if (riter->m_position 
+			- (m_ps->m_isTabPositionRelative ? 0.0f : (m_ps->m_pageMarginLeft + m_ps->m_sectionMarginLeft + m_ps->m_leftMarginByParagraphMarginChange))
+			< (m_ps->m_leftMarginByTabs + m_ps->m_textIndentByTabs + m_ps->m_textIndentByParagraphIndentChange))
+			return riter->m_position
+				- (m_ps->m_isTabPositionRelative ? 0.0f : (m_ps->m_pageMarginLeft + m_ps->m_sectionMarginLeft + m_ps->m_leftMarginByParagraphMarginChange));
+	}
+	return (std::numeric_limits<float>::max)();
+}
+
 WPXString WPXContentListener::_colorToString(const RGBSColor * color)
 {
 	WPXString tmpString;
@@ -1099,7 +1159,7 @@ WPXString WPXContentListener::_mergeColorsToString(const RGBSColor *fgColor, con
 	WPXString tmpColor;
 	RGBSColor tmpFgColor, tmpBgColor;
 
-	if (fgColor != NULL) {
+	if (fgColor) {
 		tmpFgColor.m_r = fgColor->m_r;
 		tmpFgColor.m_g = fgColor->m_g;
 		tmpFgColor.m_b = fgColor->m_b;
@@ -1109,7 +1169,7 @@ WPXString WPXContentListener::_mergeColorsToString(const RGBSColor *fgColor, con
 		tmpFgColor.m_r = tmpFgColor.m_g = tmpFgColor.m_b = 0xFF;
 		tmpFgColor.m_s = 0x64; // 100%
 	}
-	if (bgColor != NULL) {
+	if (bgColor) {
 		tmpBgColor.m_r = bgColor->m_r;
 		tmpBgColor.m_g = bgColor->m_g;
 		tmpBgColor.m_b = bgColor->m_b;
@@ -1138,7 +1198,7 @@ float WPXContentListener::_movePositionToFirstColumn(float position)
 		return position;
 	float tempSpaceRemaining = position - m_ps->m_pageMarginLeft - m_ps->m_sectionMarginLeft;
 	position -= m_ps->m_textColumns[0].m_leftGutter;
-	for (int i = 0; i < (m_ps->m_textColumns.size() - 1); i++)
+	for (int i = 0; i < (int)(m_ps->m_textColumns.size() - 1); i++)
 	{
 		if ((tempSpaceRemaining -= m_ps->m_textColumns[i].m_width - m_ps->m_textColumns[i].m_rightGutter) > 0)
 		{
