@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2002-2004 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
@@ -23,8 +23,6 @@
  * Corel Corporation or Corel Corporation Limited."
  */
 
-#include <stdlib.h>
-#include <string.h>
 #include "WPXHeader.h"
 #include "WP3Header.h"
 #include "WP5Header.h"
@@ -32,10 +30,12 @@
 #include "WP60Header.h"
 #include "WP61Header.h"
 #include "WPXFileStructure.h"
-#include "libwpd.h"
 #include "libwpd_internal.h"
+
+#include <string.h>
 	
-WPXHeader::WPXHeader(WPXInputStream *input, uint32_t documentOffset, uint8_t productType, uint8_t fileType, uint8_t majorVersion, uint8_t minorVersion, uint16_t documentEncryption) :
+WPXHeader::WPXHeader(WPXInputStream * /* input */, uint32_t documentOffset, uint8_t productType,
+			uint8_t fileType, uint8_t majorVersion, uint8_t minorVersion, uint16_t documentEncryption) :
 	m_documentOffset(documentOffset),
 	m_productType(productType),
 	m_fileType(fileType),
@@ -53,17 +53,16 @@ WPXHeader * WPXHeader::constructHeader(WPXInputStream *input)
 {
 	WPD_DEBUG_MSG(("WPXHeader::constructHeader()\n"));
 	
-	char fileMagic[4];
+	char fileMagic[4] = { 0, 0, 0, 0 };
 	/* check the magic */
 	input->seek(WPX_HEADER_MAGIC_OFFSET - input->tell(), WPX_SEEK_CUR);
-	for (int i=0; i<3 /* FIXME: && not EOF */; i++)
+	for (int i=0; i<3 && !input->atEOS(); i++)
 		fileMagic[i] = readU8(input);
-	fileMagic[3] = '\0';
 	
 	if ( strcmp(fileMagic, "WPC") )
 	{
 		WPD_DEBUG_MSG(("WordPerfect: File magic is not equal to \"WPC\"!\n"));
-		return NULL;
+		return 0;
 	}
 	
 	/* get the document pointer */
@@ -108,7 +107,7 @@ WPXHeader * WPXHeader::constructHeader(WPXInputStream *input)
 				default:
 					// unhandled file format
 					WPD_DEBUG_MSG(("WordPerfect: Unsupported major number: %d\n", majorVersion));
-					return NULL;
+					return 0;
 			}
 		case 0x2c:
 			WPD_DEBUG_MSG(("WordPerfect: Supported file type: \"MAC WP 2.0 document\"\n"));
@@ -122,13 +121,13 @@ WPXHeader * WPXHeader::constructHeader(WPXInputStream *input)
 				default:
 					// unhandled file format
 					WPD_DEBUG_MSG(("WordPerfect: Unsupported major number: %d\n", majorVersion));
-					return NULL;
+					return 0;
 			}
 		default:
 			WPD_DEBUG_MSG(("WordPerfect: Unsupported file type: %d\n", fileType));
-			return NULL;
+			return 0;
 	}
 	
 
-	return NULL;
+	return 0;
 }
