@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2002 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
@@ -23,10 +23,8 @@
  * Corel Corporation or Corel Corporation Limited."
  */
 
-#include <stdlib.h>
-#include <string.h>
 #include "WPXParser.h"
-#include "WP6Parser.h"
+#include "WPXHeader.h"
 #include "libwpd_internal.h"
 
 WPXParser::WPXParser(WPXInputStream *input, WPXHeader *header) :
