@@ -25,6 +25,8 @@
 #include "libwpd_internal.h"
 #include "WPXStream.h"
 #include <ctype.h>
+#include <locale.h>
+#include <string>
 
 uint8_t readU8(WPXInputStream *input)
 {
@@ -95,10 +97,6 @@ WPXString readCString(WPXInputStream *input)
 // the ascii map appears stupid, but we need the const 16-bit data for now
 static const uint16_t asciiMap[] =
 {
-	  0,   1,   2,   3,   4,   5,   6,   7,
-	  8,   9,  10,  11,  12,  13,  14,  15,
-	 16,  17,  18,  19,  20,  21,  22,  23,
-	 24,  25,  26,  27,  28,  29,  30,  31,
 	 32,  33,  34,  35,  36,  37,  38,  39,
 	 40,  41,  42,  43,  44,  45,  46,  47,
 	 48,  49,  50,  51,  52,  53,  54,  55,
@@ -110,23 +108,7 @@ static const uint16_t asciiMap[] =
 	 96,  97,  98,  99, 100, 101, 102, 103,
 	104, 105, 106, 107, 108, 109, 110, 111,
 	112, 113, 114, 115, 116, 117, 118, 119,
-	120, 121, 122, 123, 124, 125, 126, 127,
-	128, 129, 130, 131, 132, 133, 134, 135,
-	136, 137, 138, 139, 140, 141, 142, 143,
-	144, 145, 146, 147, 148, 149, 150, 151,
-	152, 153, 154, 155, 156, 157, 158, 159,
-	160, 161, 162, 163, 164, 165, 166, 167,
-	168, 169, 170, 171, 172, 173, 174, 175,
-	176, 177, 178, 179, 180, 181, 182, 183,
-	184, 185, 186, 187, 188, 189, 190, 191,
-	192, 193, 194, 195, 196, 197, 198, 199,
-	200, 201, 202, 203, 204, 205, 206, 207,
-	208, 209, 210, 211, 212, 213, 214, 215,
-	216, 217, 218, 219, 220, 221, 222, 223,
-	224, 225, 226, 227, 228, 229, 230, 231,
-	232, 233, 234, 235, 236, 237, 238, 239,
-	240, 241, 242, 243, 244, 245, 246, 247,
-	248, 249, 250, 251, 252, 253, 254, 255,
+	120, 121, 122, 123, 124, 125, 126
 };
 
 /* WP6 Extended Character -> Unicode (UCS2) Mappings by Ariya Hidayat <ariyahidayat@yahoo.de> for the
@@ -504,7 +486,10 @@ int extendedCharacterWP6ToUCS2(uint8_t character,
 	{
 		// if characterset == 0, we have ascii. note that this is different from the doc. body
 		// this is not documented in the file format specifications
-		*chars = &asciiMap[character];
+		if (character >= 0x20 && character < 0x7F)
+			*chars = &asciiMap[character - 0x20];
+		else
+			*chars = &asciiMap[0x00];
 		return 1;
 	}
 
@@ -514,77 +499,77 @@ int extendedCharacterWP6ToUCS2(uint8_t character,
 		if (character < WP6_NUM_MULTINATIONAL_CHARACTERS)
 			*chars = &multinationalWP6[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 		
 	case WP6_PHONETIC_SYMBOL_CHARACTER_SET:
 		if (character < WP6_NUM_PHONETIC_CHARACTERS)
 			*chars = &phoneticWP6[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP6_BOX_DRAWING_CHARACTER_SET:
 		if (character < WP6_NUM_BOX_DRAWING_CHARACTERS)
 			*chars = &boxdrawingWP6[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP6_TYPOGRAPHIC_SYMBOL_CHARACTER_SET:
 		if (character < WP6_NUM_TYPOGRAPHIC_CHARACTERS)
 			*chars = &typographicWP6[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP6_ICONIC_SYMBOL_CHARACTER_SET:
 		if (character < WP6_NUM_ICONIC_CHARACTERS)
 			*chars = &iconicWP6[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP6_MATH_SCIENTIFIC_CHARACTER_SET:
 		if (character < WP6_NUM_MATH_SCIENTIFIC_CHARACTERS)
 			*chars = &mathWP6[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP6_MATH_SCIENTIFIC_EXTENDED_CHARACTER_SET:
 		if (character < WP6_NUM_MATH_SCIENTIFIC_EXTENDED_CHARACTERS)
 			*chars = &mathextWP6[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP6_GREEK_CHARACTER_SET:
 		if (character < WP6_NUM_GREEK_CHARACTERS)
 			*chars = &greekWP6[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP6_HEBREW_CHARACTER_SET:
 		if (character < WP6_NUM_HEBREW_CHARACTERS)
 			*chars = &hebrewWP6[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP6_CYRILLIC_CHARACTER_SET:
 		if (character < WP6_NUM_CYRILLIC_CHARACTERS)
 			*chars = &cyrillicWP6[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP6_JAPANESE_CHARACTER_SET:
 		if (character < WP6_NUM_JAPANESE_CHARACTERS)
 			*chars = &japaneseWP6[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP6_TIBETAN_CHARACTER_SET:
@@ -595,7 +580,7 @@ int extendedCharacterWP6ToUCS2(uint8_t character,
 			return i;
 		}
 		else {
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 			return 1;
 		}
 
@@ -603,19 +588,19 @@ int extendedCharacterWP6ToUCS2(uint8_t character,
 		if (character < WP6_NUM_ARABIC_CHARACTERS)
 			*chars = &arabicWP6[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP6_ARABIC_SCRIPT_CHARACTER_SET:
 		if (character < WP6_NUM_ARABIC_SCRIPT_CHARACTERS)
 			*chars = &arabicScriptWP6[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 	}
 
 	// last resort: return whitespace
-	*chars = &asciiMap[32];
+	*chars = &asciiMap[0x00];
 	return 1;
 }
 
@@ -847,7 +832,10 @@ int extendedCharacterWP5ToUCS2(uint8_t character,
 	{
 		// if characterset == 0, we have ascii. note that this is different from the doc. body
 		// this is not documented in the file format specifications
-		*chars = &asciiMap[character];
+		if (character >= 0x20 && character < 0x7F)
+			*chars = &asciiMap[character - 0x20];
+		else
+			*chars = &asciiMap[0x00];
 		return 1;
 	}
 
@@ -857,80 +845,82 @@ int extendedCharacterWP5ToUCS2(uint8_t character,
 		if (character < WP5_NUM_INTERNATIONAL_1_CHARACTERS)
 			*chars = &international1WP5[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP5_INTERNATIONAL_2_CHARACTER_SET:
 		if (character < WP5_NUM_INTERNATIONAL_2_CHARACTERS)
 			*chars = &international2WP5[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP5_BOX_DRAWING_CHARACTER_SET:
 		if (character < WP5_NUM_BOX_DRAWING_CHARACTERS)
 			*chars = &boxdrawingWP5[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP5_TYPOGRAPHIC_SYMBOL_CHARACTER_SET:
 		if (character < WP5_NUM_TYPOGRAPHIC_CHARACTERS)
 			*chars = &typographicWP5[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP5_ICONIC_SYMBOL_CHARACTER_SET:
 		if (character < WP5_NUM_ICONIC_CHARACTERS)
 			*chars = &iconicWP5[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP5_MATH_SCIENTIFIC_CHARACTER_SET:
 		if (character < WP5_NUM_MATH_SCIENTIFIC_CHARACTERS)
 			*chars = &mathWP5[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP5_MATH_SCIENTIFIC_EXTENDED_CHARACTER_SET:
 		if (character < WP5_NUM_MATH_SCIENTIFIC_EXTENDED_CHARACTERS)
 			*chars = &mathextWP5[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP5_GREEK_CHARACTER_SET:
 		if (character < WP5_NUM_GREEK_CHARACTERS)
 			*chars = &greekWP5[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP5_HEBREW_CHARACTER_SET:
 		if (character < WP5_NUM_HEBREW_CHARACTERS)
 			*chars = &hebrewWP5[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 
 	case WP5_CYRILLIC_CHARACTER_SET:
 		if (character < WP5_NUM_CYRILLIC_CHARACTERS)
 			*chars = &cyrillicWP5[character];
-			return 1;
+		else
+			*chars = &asciiMap[0x00];
+		return 1;
 
 	case WP5_JAPANESE_CHARACTER_SET:
 		if (character < WP5_NUM_JAPANESE_CHARACTERS)
 			*chars = &japaneseWP5[character];
 		else
-			*chars = &asciiMap[32];
+			*chars = &asciiMap[0x00];
 		return 1;
 	}
 
 	// last resort: return whitespace
-	*chars = &asciiMap[32];
+	*chars = &asciiMap[0x00];
 	return 1;
 }
 
@@ -969,31 +959,11 @@ _RGBSColor::_RGBSColor()
 }
 
 _RGBSColor::_RGBSColor(uint16_t red, uint16_t green, uint16_t blue)
-	:	m_r(0),
-		m_g(0),
-		m_b(0),
-		m_s(0)
+	:	m_r((red >> 8) & 0xFF),
+		m_g((green >> 8) & 0xFF),
+		m_b((blue >> 8) & 0xFF),
+		m_s(100)
 {
-	unsigned int minRGB = red;
-	if (minRGB > green)
-		minRGB = green;
-	if (minRGB > blue)
-		minRGB = blue;
-		
-	if (minRGB >= 65535)
-	{
-		m_r = 255;
-		m_g = 255;
-		m_b = 255;
-		m_s = 100;
-	}
-	else
-	{
-		m_r = (uint8_t)rint(255*((double)(red - minRGB))/((double)(65535 - minRGB)));
-		m_g = (uint8_t)rint(255*((double)(green - minRGB))/((double)(65535 - minRGB)));
-		m_b = (uint8_t)rint(255*((double)(blue - minRGB))/((double)(65535 - minRGB)));
-		m_s = (uint8_t)rint(100*((double)(65535 - minRGB))/(double)65535);
-	}		
 }
 
 _WPXTabStop::_WPXTabStop(float position, WPXTabAlignment alignment, uint16_t leaderCharacter, uint8_t leaderNumSpaces)
@@ -1173,3 +1143,20 @@ const uint16_t macintoshCharacterMap[] =
   0xf8ff, 0x00d2, 0x00da, 0x00db, 0x00d9, 0x0131, 0x02c6, 0x02dc,
   0x00af, 0x02d8, 0x02d9, 0x02da, 0x00b8, 0x02dd, 0x02db, 0x02c7
 };
+
+WPXString doubleToString(const double value)
+{
+  WPXString tempString;
+  tempString.sprintf("%.4f", value);
+  std::string decimalPoint(localeconv()->decimal_point);
+  if ((decimalPoint.size() == 0) || (decimalPoint == "."))
+    return tempString;
+  std::string stringValue(tempString.cstr());
+  if (!stringValue.empty())
+  {
+    std::string::size_type pos;
+    while ((pos = stringValue.find(decimalPoint)) != std::string::npos)
+          stringValue.replace(pos,decimalPoint.size(),".");
+  }
+  return WPXString(stringValue.c_str());
+}
